﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.Libs.Drawing;
using Diplo.DrawAlgorithms.Lib;
using Diplo.GraphObjects;
using Diplo.Configs;
using Diplo.Libs.Geometry;
using Diplo.Libs;

namespace Diplo.DrawAlgorithms
{
    /// <summary>
    /// Kresliaci algoritmus A*
    /// </summary>
    class AStar : DrawInterface
    {

        /// <summary>
        /// <para>Zoznam bodov, ktoré tvoria cestu.</para>
        /// </summary>
        private Stack<Point> path = new Stack<Point>();

        /// <summary>
        /// 
        /// </summary>
        private Stack<Point> polygon = new Stack<Point>();

        /// <summary>
        /// <para>Určuje granularitu pri hľadaní možných susedov. Čím je parameter menší,
        /// tým je cesta kvalitnejšia (hladšia) ale zároveň sa vyžaduje viac výpočtov pri hľadaní cesty.</para>
        /// </summary>
        public int granul = 5;

        /// <summary>
        /// Ak je parameter zapnutý, 
        /// </summary>
        Boolean allowCompact = true;

        /// <summary>
        /// Ochrana - ochrana proti zacykleniu
        /// </summary>
        private int t = 0;
        private int r = 0;

        /// <summary>
        /// <para>Hľadanie cesty pomocou A*</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="graf">Inštancia grafu</param>
        /// <param name="start">Štart bod</param>
        /// <param name="end">Koncový bod</param>
        /// <param name="Polygon">Poligon</param>
        /// <returns>Cesta, množina bodov</returns>
        public Stack<Point> drawIntoPolygon(Graphics g, Graph graf, Point start, Point end, Stack<Point> Polygon)
        {
            //polygon = Polygon;
            StackHelper.copyStackToStack(Polygon, this.polygon);
            path = this.draw(g, graf, start, end, 600);
            this.polygon.Clear();
            return path;
        }

        /// <summary>
        /// <para>Hľadanie cesty pomocou A*, cesta nemôže výjsť z poligónu (oblasti)</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="graf">Inštancia grafu</param>
        /// <param name="start">Štart bod</param>
        /// <param name="end">Koncový bod</param>
        /// <param name="fieldSize">Veľkost</param>
        /// <returns>Cesta, množina bodov</returns>
        public Stack<Point> draw(Graphics g, Graph graf, Point start, Point end, int fieldSize)
        {
            path.Clear();
            fieldSize = 1000;

            List<Point> closedset = new List<Point>();
            List<Point> openset = new List<Point>();
            openset.Add(start);

            
            int[,,] came_from = new int[fieldSize ,fieldSize, 2];

            int[,] g_score = new int[fieldSize ,fieldSize];
            int[,] h_score = new int[fieldSize, fieldSize];
            int[,] f_score = new int[fieldSize, fieldSize];

            g_score[start.X, start.Y] = 0;
            h_score[start.X, start.Y] = TwoPoints.manhatan_distance(start, end);
            f_score[start.X, start.Y] = g_score[start.X, start.Y] + h_score[start.X, start.Y];


            Point current;

            
            r = 0;
            //DrawPoint.drawHighligthedPoint(g, start, "Blue", 15);
            //DrawPoint.drawHighligthedPoint(g, end, "Pink", 15);
            
            while (openset.Count() != 0 && r <= 3000)
            //while (openset.Count() != 0) 
            {   
                r++;

                current = this.findInOpenSetWithLowestFScore(openset, f_score, end);

                if (current.X != -1 && current.Y != -1)
                {
                    // ak uz som v ciely
                    if (current.X >= end.X - this.granul * 2 && current.X <= end.X + this.granul * 2 &&
                        current.Y >= end.Y - this.granul * 2 && current.Y <= end.Y + this.granul * 2)
                    {
                        t = 0;
                        if (came_from[end.X, end.Y, 0] == 0 && came_from[end.X, end.Y, 1] == 0) { }
                        else
                        {
                            this.reconstruct_path(g, came_from, new Point(came_from[end.X, end.Y, 0], came_from[end.X, end.Y, 1]), start);
                        }

                        path = this.compactPath(path, end);
                        path.Push(start);
                        
                        return path;
                    }

                    // zmazeme z opensetu current vrchol - aby sme ho viac krat neiterovali
                    ListHelper.removePointFromList(openset, current);
                    closedset.Add(current);

                    int[,] neigh = AStarNeighbor.getNeighborNodes(g, graf, start, end, current, fieldSize, this.granul, polygon);
                    for (int i = 0; i < 8; i++)
                    {

                        if (neigh[i, 0] != -1 && neigh[i, 1] != -1 && neigh[i, 0] < Config.fieldSize && neigh[i, 1] < Config.fieldSize) // ak je to "dobry sused"
                        {
                            // ak je uz v closeliste, ideme dalej
                            if (ListHelper.isPointInList(closedset, new Point(neigh[i, 0], neigh[i, 1]))) { continue; }

                            int distance = Math.Abs(current.X - neigh[i, 0]) + Math.Abs(current.Y - neigh[i, 1]);
                            int tentative_g_score = g_score[current.X, current.Y] + distance;
                            Boolean tentative_is_better;

                            if (!ListHelper.isPointInList(openset, new Point(neigh[i, 0], neigh[i, 1])))
                            {
                                openset.Add(new Point(neigh[i, 0], neigh[i, 1]));
                                tentative_is_better = true;
                            }
                            else if (tentative_g_score < g_score[neigh[i, 0], neigh[i, 1]])
                            {
                                tentative_is_better = true;
                            }
                            else
                            {
                                tentative_is_better = false;
                            }

                            if (tentative_is_better)
                            {

                                if (neigh[i, 0] >= end.X - this.granul * 2 && neigh[i, 0] <= end.X + this.granul * 2 &&
                                    neigh[i, 1] >= end.Y - this.granul * 2 && neigh[i, 1] <= end.Y + this.granul * 2)
                                {
                                    came_from[end.X, end.Y, 0] = current.X;
                                    came_from[end.X, end.Y, 1] = current.Y;
                                }
                                else
                                {
                                    came_from[neigh[i, 0], neigh[i, 1], 0] = current.X;
                                    came_from[neigh[i, 0], neigh[i, 1], 1] = current.Y;


                                }

                                if (this.polygon.Count == 0)
                                {
                                    //DrawPoint.drawPoint(g, current, "Green");
                                }
                                else
                                {
                                    //DrawPoint.drawPoint(g, current, "Pink");
                                }

                                g_score[neigh[i, 0], neigh[i, 1]] = tentative_g_score;
                                h_score[neigh[i, 0], neigh[i, 1]] = TwoPoints.manhatan_distance(new Point(neigh[i, 0], neigh[i, 1]), end);
                                f_score[neigh[i, 0], neigh[i, 1]] = g_score[neigh[i, 0], neigh[i, 1]] + h_score[neigh[i, 0], neigh[i, 1]];

                            }

                        }
                    }

                }
            }

            return new Stack<Point>();
        }

        
        /// <summary>
        /// <para>RekonŠtrukcia cesty. A* ukladá najlepšiu cestu medzi dvoma bodmi do poľa (came_from). 
        /// V poli sa ukladá referencia na nasledujúci bod na ceste (znova do pola).
        /// Metóda postupuje reverzne od cieľa, v poly, až kým nedosiahne začiatok.</para> 
        /// <para>Pozn. : parameter T - ochrana proti zacikleniu</para>
        /// </summary>
        /// <param name="g"></param>
        /// <param name="came_from"></param>
        /// <param name="current_node"></param>
        /// <param name="start"></param>
        private void reconstruct_path(Graphics g, int[,,] came_from, Point current_node, Point start){
            t++;
            if (t > 1000) { return; }

		    if(start.X == current_node.X && start.Y == current_node.Y){
                // koniec hladania    
		    }
		    else{
                
                this.reconstruct_path(g, came_from, new Point(came_from[current_node.X, current_node.Y, 0], came_from[current_node.X, current_node.Y, 1]), start);
                this.path.Push(current_node);
		    }
	    }

        /// <summary>
        /// <para>Ak dva body v ceste (ktoré su vedľa seba v množine path) ležia na tej istej priamke,
        /// bude druhý bod z výslednej cesty odobraný. Hľadáme len významne body, v ktorých cesta mení smer.</para>
        /// </summary>
        /// <param name="path">Cesta, postupnosť bodov</param>
        /// <param name="end">Cieľ</param>
        /// <returns></returns>
        private Stack<Point> compactPath(Stack<Point> path, Point end)
        {
            Stack<Point> newPath = new Stack<Point>();
            newPath.Push(end);

            int smer = -1;
            int novySmer = -1;
            for (int i = 1; i < path.Count;i++)
            {
                Point p1 = path.ElementAt(i-1);
                Point p2 = path.ElementAt(i);

                if (p1.Y < p2.Y)
                {
                    if (p1.X < p2.X) { novySmer = 1; }
                    else if (p1.X == p2.X) { novySmer = 2; }
                    else { novySmer = 3; }
                }
                else if (p1.Y == p2.Y)
                {
                    if (p1.X < p2.X) { novySmer = 4; }
                    else { novySmer = 5; }
                }
                else
                {
                    if (p1.X < p2.X) { novySmer = 6; }
                    else if (p1.X == p2.X) { novySmer = 7; }
                    else { novySmer = 8; }
                }

                if (smer != novySmer || !allowCompact)
                { 
                    newPath.Push(new Point(path.ElementAt(i).X, path.ElementAt(i).Y));
                }

                smer = novySmer;
            }


            return newPath;
        }

        /// <summary>
        /// <para>V množine openset nájde bod, ktorý ma najmenšie FScore</para>
        /// </summary>
        /// <param name="openset">Množina bodov - kandidátov</param>
        /// <param name="f_score">FScore - hodnota bodu, FScore</param>
        /// <param name="goal">Cieľ</param>
        /// <returns></returns>
        private Point findInOpenSetWithLowestFScore(List<Point> openset, int[,] f_score, Point goal)
        {
            int lowestIndex = -1;
            int lowestScore = 99999999;

            for (int i = 0; i < openset.Count;i++)
            {
                if (openset.ElementAt(i).X < Config.fieldSize && openset.ElementAt(i).Y < Config.fieldSize)
                {
                    if (f_score[openset.ElementAt(i).X, openset.ElementAt(i).Y] < lowestScore)
                    {
                        lowestIndex = i;
                        lowestScore = f_score[openset.ElementAt(i).X, openset.ElementAt(i).Y];
                    }
                }
                
            }

            int lowestMDScore = Int32.MaxValue;
            for (int i = 0; i < openset.Count; i++)
            {
                if (openset.ElementAt(i).X < Config.fieldSize && openset.ElementAt(i).Y < Config.fieldSize)
                {

                    Point p = openset.ElementAt(i);
                    if (f_score[p.X, p.Y] == lowestScore)
                    {

                        if (TwoPoints.manhatan_distance(new Point(p.X, p.Y), goal) < lowestMDScore)
                        {
                            lowestIndex = i;
                            lowestMDScore = TwoPoints.manhatan_distance(new Point(p.X, p.Y), goal);
                        }
                    }
                }
            }


            if (lowestIndex != -1)
            {
                return openset.ElementAt(lowestIndex);
            }

            return new Point(-1, -1);
        }

    }
}
