﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.GraphObjects;

namespace Diplo.DrawAlgorithms
{
    /// <summary>
    /// Lee argoritmus (nedkonceny, nepracuje spravne)
    /// </summary>
    class Lee : DrawInterface
    {
        /* Private premenne */
        private Random r = new Random();
        private Graphics g = null;

        private List<Vertex> visited = new List<Vertex>();
        public Stack<Point> path = new Stack<Point>();
        private Graph graf = new Graph();
        
        private int startXOriginal = -1;
        private int startYOriginal = -1;
        private int finishXOriginal = -1;
        private int finishYOriginal = -1;
        
        private int smer = -1;
        private int novySmer = -1;

        private int counter = 0;

        /* konfiguracie */
        private int radius = 10;
        private int vertexArea = 10;

        /// <summary>
        /// <para>Nájdenie cesty z bodu START do bodu END </para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="graf">Inštancia grafu</param>
        /// <param name="start">Štart bod</param>
        /// <param name="end">Koncový bod</param>
        /// <param name="fieldSize">Veľkosť plochy pre vrchol</param>
        /// <returns>Cesta, množina bodov</returns>
        public Stack<Point> draw(Graphics g, Graph graf, Point start, Point end, int fieldSize)
        {
            // inicializacie
            this.path.Clear();
            this.visited.Clear();
            this.graf = graf;
            this.counter = 0;
            this.startXOriginal = start.X;
            this.startXOriginal = start.Y;
            this.finishXOriginal = end.X;
            this.finishYOriginal = end.Y;

            // vytvorenie cesty
            this.path.Push(end);
            this.constructPath(start.X, start.Y, end.X, end.Y, r);
            this.path.Push(start);

            return this.path;
        }

        /// <summary>Lee-Algoritmus (rekurizvne) hlada cestu.</summary>
        /// <param name="x">Startovný vrchol - súradnica X</param>
        /// <param name="y">Startovný vrchol - súradnica Y</param>
        /// <param name="finishX">Startovný vrchol - súradnica X</param>
        /// <param name="finishY">Startovný vrchol - súradnica Y</param>
        /// <param name="r">Inštancia RANDOM</param>
        public void constructPath(int x, int y, int finishX, int finishY, Random r)
        {
            counter++;
            // urcime, kedy ma davat pozor na vrchlo, ktore sa mozu vyskytnut v ceste
            bool checkForNodes = true;
            if (x == finishX && y == finishY) { checkForNodes = false; }
            //if (counter<4) { checkForNodes = false; }

            int[,] neigh = getValuedNeightboors(g, visited, x, y, checkForNodes); // ziskame susedov

            // co ak je sused uz ciel ?
            bool finish = false;
            for (int i = 0; i < 4; i++)
            {
                if (checkForFinishInRadius(neigh[i, 0], neigh[i, 1], finishX, finishY))
                {
                    finish = true; // sme blizko ciela
                }
            }

            // ak este nie sme pri konci - druhy vrchol hrany
            if (!finish)
            {
                int[] values = new int[8];
                for (int i = 0; i < 8; i++) // ohodnotime hrany - cislo > -1 ak je to platny sused || -1 ak je to neplatny sused
                {
                    if (neigh[i, 0] != -1 && neigh[i, 1] != -1)
                    {
                        values[i] = heuresticManhattan(neigh[i, 0], neigh[i, 1], finishX, finishY);
                    }
                    else
                    {
                        values[i] = -1;
                    }
                }

                // vyberieme minimum - najlepsi kandidat kam ist
                int minId = -1;
                int minScore = values[0];
                for (int i = 0; i < 8; i++)
                {
                    if (neigh[i, 0] != -1 && neigh[i, 1] != -1)
                    {
                        if (minId == -1)
                        {
                            minScore = values[i];
                            minId = i;
                        }

                        if (values[i] < minScore && values[i] != -1)
                        {
                            minScore = values[i];
                            minId = i;
                        }
                    }
                }

                // ak mame minimum a nespravili sme viac ako 1000 krokov (aby sa to velmi rekurzivne nevolalo ...)
                if (minId != -1 && counter < 1000)
                {
                    if (neigh[minId, 0] != -1 && neigh[minId, 1] != -1)
                    {

                        visited.Add(new Vertex(counter, neigh[minId, 0], neigh[minId, 1], r));
                        constructPath(neigh[minId, 0], neigh[minId, 1], finishX, finishY, r);


                        // ! Mentalna mapa hran
                        novySmer = minId;
                        if (smer != novySmer)
                        {
                            smer = novySmer;
                            this.path.Push(new Point(neigh[minId, 0], neigh[minId, 1]));
                        }


                    }
                }
                else
                {
                    System.Console.WriteLine("HRANA SA NENASLA!!!");
                }

            }
            else
            {
                //LeeMM.vertices.Add(new Point(finishX, finishY));

            }


        }


        /*
         * 
         * Heuristiky
         * 
         */

        // fnkcia, na kontrolu, ci nie je v 'oblasti-vertexArea' nejaky vrchol - takyto vrchol musime obist
        public bool nodeDetected(Graphics g, int x, int y)
        {
            foreach (Vertex v in graf.vertices)
            {
                // vrchol nemoze byt v stvorci, ktory patri uz rozmiestnenym vrcholom grafu

                int x1 = (int)Math.Round(v.pos[0]);
                int y1 = (int)Math.Round(v.pos[1]);

                // ak je to START vrchol, tak to je OK
                Boolean startVertex = false;
                if (x1 == this.startXOriginal && y1 == this.startYOriginal)
                {
                    startVertex = true;
                }

                // ak je to FINISH vrchol
                Boolean finishVertex = false;
                if (x1 == this.finishXOriginal && y1 == this.finishYOriginal)
                {
                    finishVertex = true;
                }

                if (x == x1 && y == y1) { }
                else
                {

                    int left = x1 - vertexArea;
                    int up = y1 - vertexArea;
                    int right = x1 + v.size + vertexArea;
                    int down = y1 + v.size + vertexArea;

                    if ((left < x && right > x) && (up < y && down > y) && !startVertex && !finishVertex)
                    {
                        return true;
                    }

                }



            }

            return false;
        }

        // ak uz mame v dosahu vrchol, vratime TRUE
        private bool checkForFinishInRadius(int x, int y, int finishX, int finishY)
        {
            if ((x > finishX - radius - vertexArea * 2 && x < finishX + radius + vertexArea * 2) && (y > finishY - radius - vertexArea * 2 && y < finishY + radius + vertexArea * 2))
            {
                return true;
            }

            if (Math.Abs(x - finishX) + Math.Abs(y - finishY) < vertexArea * 2)
            {
                return true;
            }

            return false;
        }



        // heuristika - odhad vzdialenosti medzi bodmi
        private int heuresticManhattan(int x1, int y1, int x2, int y2)
        {
            int res = 0;
            //System.Console.WriteLine(x1+"|"+y1+" ||| "+x2+"|"+y2);
            res = Math.Abs(x1 - x2);
            res += Math.Abs(y1 - y2);

            return res;
        }



        // funkcia najde susedov
        private int[,] getValuedNeightboors(Graphics g, List<Vertex> visited, int x, int y, bool checkForNode)
        {
            int[,] res = new int[8, 2];

            int count = 0;
            for (int i = x - radius; i <= x + radius; i = i + radius)
            {
                for (int j = y - radius; j <= y + radius; j = j + radius)
                {
                    if (i == x && j == y) { }
                    else
                    {
                        bool nodeDet = false;
                        if (checkForNode)
                        {
                            nodeDet = nodeDetected(g, i, j);
                        }

                        if (!containsListVertex(visited, i, j) && !nodeDet && (i > 0) && (j > 0))
                        {
                            res[count, 0] = i;
                            res[count, 1] = j;
                        }
                        else
                        {
                            res[count, 0] = -1;
                            res[count, 1] = -1;
                        }

                        count++;
                    }
                }
            }

            return res;

        }





        /**
         * Pomocne funkcie
         */

        private bool containsListVertex(List<Vertex> visited, int x, int y)
        {
            foreach (Vertex v in visited)
            {
                if (v.pos[0] == x && v.pos[1] == y) { return true; }
            }

            return false;
        }


        public Stack<Point> findPath(int x, int y, int finishX, int finishY)
        {
            Random r = new Random();

            this.path.Clear();
            this.constructPath(x, y, finishX, finishY, r);
            return this.path;
        }


        /// <summary>
        /// <para>Nájdenie cesty z bodu START do bodu END. Pri hľadaní, cesta nemôže výjsť z poligónu (oblasti)</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="graf">Inštancia grafu</param>
        /// <param name="start">Štart bod</param>
        /// <param name="end">Koncový bod</param>
        /// <param name="Polygon">Poligon - Koridor</param>
        /// <returns>Cesta, množina bodov</returns>
        public Stack<Point> drawIntoPolygon(Graphics g, Graph graf, Point start, Point end, Stack<Point> Polygon)
        {
            
            return this.draw(g, graf, start, end, 600);
        }
    }
}
