﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.MentalMap.CoridorObjects;

namespace Diplo.Libs.Drawing
{
    /// <summary>
    /// Trieda pre kreslenie ciet
    /// </summary>
    class DrawPath
    {

        /// <summary>
        /// <para>Jednoduche vykreslenie cesty</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="path">Cesta, postupnosť bodov</param>
        internal static void simple(Graphics g, Stack<System.Drawing.Point> path)
        {
            Point prevPoint = new Point(0, 0);
            int a = 0;
            foreach (Point p in path)
            {

                if (a == 0 || a - 1 == path.Count)
                {
                    prevPoint = new Point(p.X, p.Y);
                    g.FillEllipse(new SolidBrush(Color.Green), p.X - 3, p.Y - 3, 6, 6);
                }
                else
                {
                    g.DrawLine(System.Drawing.Pens.Green, prevPoint.X, prevPoint.Y, p.X, p.Y);
                    prevPoint = new Point(p.X, p.Y);
                    g.FillEllipse(new SolidBrush(Color.Green), p.X - 3, p.Y - 3, 6, 6);
                }

                a++;

            }
        }

        /// <summary>
        /// <para>Jednoduche vykreslenie cesty</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="path">Cesta, postupnosť bodov</param>
        internal static void simple(Graphics g, List<CoridorPoint> path)
        {
            CoridorPoint prevPoint = new CoridorPoint(0, 0);
            int a = 0;
            foreach (CoridorPoint p in path)
            {
                if (a == 0 || a - 1 == path.Count)
                {
                    prevPoint = new CoridorPoint(p.X, p.Y);
                }
                else
                {
                    g.DrawLine(System.Drawing.Pens.Green, prevPoint.X, prevPoint.Y, p.X, p.Y);
                    prevPoint = new CoridorPoint(p.X, p.Y);
                    g.FillEllipse(new SolidBrush(Color.Green), p.X - 3, p.Y - 3, 6, 6);
                }

                a++;
            }
        }

        /// <summary>
        /// <para>Jednoduche vykreslenie cesty - RUŽOVÁ</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="path">Cesta, postupnosť bodov</param>
        internal static void pink(Graphics g, Stack<Point> path)
        {
            Point prevPoint = new Point(0, 0);
            int a = 0;
            foreach (Point p in path)
            {
                if (a == 0 || a - 1 == path.Count)
                {
                    prevPoint = new Point(p.X, p.Y);
                }
                else
                {
                    g.DrawLine(System.Drawing.Pens.Violet, prevPoint.X, prevPoint.Y, p.X, p.Y);
                    prevPoint = new Point(p.X, p.Y);
                    g.FillEllipse(new SolidBrush(Color.Violet), p.X - 3, p.Y - 3, 6, 6);
                }

                a++;
            }
        }

        /// <summary>
        /// <para>Jednoduche vykreslenie cesty - ČERVENÁ</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="path">Cesta, postupnosť bodov</param>
        internal static void red(Graphics g, Stack<Point> path)
        {
            Point prevPoint = new Point(0, 0);
            int a = 0;
            foreach (Point p in path)
            {
                if (a == 0 || a - 1 == path.Count)
                {
                    prevPoint = new Point(p.X, p.Y);
                }
                else
                {
                    g.DrawLine(System.Drawing.Pens.Red, prevPoint.X, prevPoint.Y, p.X, p.Y);
                    prevPoint = new Point(p.X, p.Y);
                    g.FillEllipse(new SolidBrush(Color.Red), p.X - 5, p.Y - 5, 10, 10);
                }

                a++;
            }
        }

        /// <summary>
        /// <para>Jednoduche vykreslenie cesty - MODRÁ</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="path">Cesta, postupnosť bodov</param>
        internal static void blue(Graphics g, Stack<Point> path)
        {
            Point prevPoint = new Point(0, 0);
            int a = 0;
            foreach (Point p in path)
            {
                if (a == 0 || a - 1 == path.Count)
                {
                    prevPoint = new Point(p.X, p.Y);
                }
                else
                {
                    g.DrawLine(System.Drawing.Pens.Blue, prevPoint.X, prevPoint.Y, p.X, p.Y);
                    prevPoint = new Point(p.X, p.Y);
                    g.FillEllipse(new SolidBrush(Color.Blue), p.X - 5, p.Y - 5, 10, 10);
                }

                a++;
            }

            // ++ posledny s prvym
            if (path.Count >= 2)
            {
                g.DrawLine(System.Drawing.Pens.Blue, path.First().X, path.First().Y,
                    path.Last().X, path.Last().Y);
                g.FillEllipse(new SolidBrush(Color.Blue), path.First().X - 5, path.First().Y - 5, 10, 10);
            }


        }

        /// <summary>
        /// <para>Jednoduche vykreslenie cesty - ZELENÁ</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="path">Cesta, postupnosť bodov</param>
        internal static void green(Graphics g, Stack<Point> path)
        {
            Point prevPoint = new Point(0, 0);
            int a = 0;
            foreach (Point p in path)
            {
                if (a == 0 || a - 1 == path.Count)
                {
                    prevPoint = new Point(p.X, p.Y);
                }
                else
                {
                    g.DrawLine(System.Drawing.Pens.Green, prevPoint.X, prevPoint.Y, p.X, p.Y);
                    prevPoint = new Point(p.X, p.Y);
                    g.FillEllipse(new SolidBrush(Color.Green), p.X - 5, p.Y - 5, 10, 10);
                }

                a++;
            }

            // ++ posledny s prvym
            if (path.Count >= 2)
            {
                g.DrawLine(System.Drawing.Pens.Green, path.First().X, path.First().Y,
                    path.Last().X, path.Last().Y);
                g.FillEllipse(new SolidBrush(Color.Green), path.First().X - 5, path.First().Y - 5, 10, 10);
            }


        }

        
    }
}
