﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.MentalMap.CoridorObjects;
using Diplo.Libs.Drawing;

namespace Diplo.Libs.Geometry.AlphaHull
{
    /// <summary>
    /// Post-procesing alfa hrán
    /// </summary>
    class AlphaEdgesToPointSequenceConvertor
    {
        private static Boolean showPoints = false;
        private static Boolean showFiltredPoints = false;
        private static Boolean coloredPoints = false;

        /// <summary>
        /// Metoda, ktora dostane dve "polovice" z vsetkych vrcholov, ktore mozdu tvorit obal. 
        /// V metode sa nasledne vyfiltruju tie vrcholy, ktore sa v obale nenachadzaju a daju do usporiadania ako v polygone
        /// Na vystupe je stack tcyhto usporiadanych bodov.
        /// </summary>
        /// <param name="g"></param>
        /// <param name="poly"></param>
        /// <param name="edges"></param>
        /// <param name="edgeBoxes"></param>
        /// <param name="coridorPolyLeft"></param>
        /// <param name="coridorPolyRight"></param>
        /// <returns></returns>
        public static Stack<System.Drawing.Point> processAlfaEdgesToPoints(Graphics g, Stack<System.Drawing.Point> poly, List<Diplo.Libs.Geometry.AlphaHull.Geometry.Edge> edges, 
            Stack<Coridor> edgeBoxes, Stack<CoridorPoint> coridorPolyLeft, Stack<CoridorPoint> coridorPolyRight)
        {
            // spracujeme obe polovice
            List<CoridorPoint> points1 = convert(g, coridorPolyLeft, poly, edges, edgeBoxes);
            List<CoridorPoint> points2 = convert(g, coridorPolyRight, poly, edges, edgeBoxes);

            // prekopirujeme do stacku
            Stack<System.Drawing.Point> polyall = new Stack<System.Drawing.Point>();
            int check = 0;
            foreach (CoridorPoint p in points1)
            {
               polyall.Push(new System.Drawing.Point(p.X, p.Y));
                check++;
            }
            foreach (CoridorPoint p in points2)
            {
                polyall.Push(new System.Drawing.Point(p.X, p.Y));
            }

            return polyall;
        }


        /// <summary>
        /// Metoda spracuje "polovicu" bodov pre obal. Najprv filtruje body, ktore nie su krajnymi bodmi ziadnej z alfa hran.
        /// Potom spaja najblizsie body.
        /// Body su vratene v usporiadani, ako by sa spajali v polygone
        /// !! bod v edgepoints - last ... 1 je 2
        /// </summary>
        /// <param name="g">Graficky komponent</param>
        /// <param name="pointsOfHalfHull">Body jednej "polovice" obalu (vsetky)</param>
        /// <param name="pointsAll"></param>
        /// <param name="edges">alfa hrany</param>
        /// <param name="edgeBoxes"></param>
        /// <returns>Usporiadane body "polivice" poligonu</returns>
        public static List<CoridorPoint> convert(Graphics g, Stack<CoridorPoint> pointsOfHalfHull, 
            Stack<System.Drawing.Point> pointsAll, List<Diplo.Libs.Geometry.AlphaHull.Geometry.Edge> edges, Stack<Coridor> edgeBoxes)
        {
            List<CoridorPoint> unsertedPointSequence = new List<CoridorPoint>();
            List<CoridorPoint> pointSequence = new List<CoridorPoint>();

            // do vyslednej mnoziny bodov si pridame iba tie, ktore su krajne body niektorej z alfa hran
            foreach (CoridorPoint p in pointsOfHalfHull)
            {
                if (showFiltredPoints) { DrawPoint.drawPoint(g, p, "Pink"); }
                
                if (isPointInSomeEdge(p, pointsAll, edges))
                {
                    unsertedPointSequence.Add(p);
                }
            }

            if (unsertedPointSequence.Count > 0)
            {
                pointSequence.Add(unsertedPointSequence.ElementAt(0));
                CoridorPoint oldPoint = unsertedPointSequence.ElementAt(0);
                // utriedime pole bodov (najblizsie k sebe )
                for (int i = 1; i < unsertedPointSequence.Count; i++)
                {

                    int minId = -1;
                    double mindist = 9999999;
                    for (int j = 1; j < unsertedPointSequence.Count; j++)
                    {
                        CoridorPoint p1 = unsertedPointSequence.ElementAt(j);

                        double dist = TwoPoints.manhatan_distance(oldPoint, p1);
                        if (dist < mindist && !ListHelper.isPointInList(pointSequence, p1))
                        {
                            mindist = dist;
                            minId = j;
                        }

                    }

                    if (minId != -1)
                    {
                        pointSequence.Add(unsertedPointSequence.ElementAt(minId));
                        oldPoint = unsertedPointSequence.ElementAt(minId);

                        // ak je to bod prveho / posledneho .. koncime
                        if ((edgeBoxes.First().points.ElementAt(2).X == oldPoint.X && edgeBoxes.First().points.ElementAt(2).Y == oldPoint.Y) ||
                           (edgeBoxes.First().points.ElementAt(1).X == oldPoint.X && edgeBoxes.First().points.ElementAt(1).Y == oldPoint.Y))
                        {
                            break;
                        }
                    }

                }
            }
            


            if (showPoints)
            {
                int c = 0;
                for (int i = 0; i < pointSequence.Count; i++)
                {
                    CoridorPoint p = pointSequence.ElementAt(i);

                    if (coloredPoints)
                    {
                        if (c % 3 == 0) { DrawPoint.drawPoint(g, new System.Drawing.Point(p.X, p.Y), "Red"); }
                        if (c % 3 == 1) { DrawPoint.drawPoint(g, new System.Drawing.Point(p.X, p.Y), "Blue"); }
                        if (c % 3 == 2) { DrawPoint.drawPoint(g, new System.Drawing.Point(p.X, p.Y), "Green"); }
                    }
                    else
                    {
                        DrawPoint.drawPoint(g, new System.Drawing.Point(p.X, p.Y), "Red");
                    }
                    

                    //if (c % 4 == 3) { DrawPoint.drawPoint(g, p, "Pink"); }
                    c++;


                }
            }
            

            return pointSequence;
        }


        public static Boolean isPointInEdgeWithPoint(Stack<System.Drawing.Point> pointsAll, List<Diplo.Libs.Geometry.AlphaHull.Geometry.Edge> edges, CoridorPoint p1, CoridorPoint p2)
        {
            //Console.WriteLine(p1.X + "| " + p1.Y + "|||" + p2.X + "|" + p2.Y);
            foreach (Diplo.Libs.Geometry.AlphaHull.Geometry.Edge edge in edges)
            {
                System.Drawing.Point a1 = pointsAll.ElementAt(edge.p1);
                System.Drawing.Point a2 = pointsAll.ElementAt(edge.p2);
                //Console.WriteLine(a1.X + "| "+ a1.Y + "|||"+ a2.X + "|" + a2.Y);
                if((a1.X == p1.X && a1.Y == p1.Y && a2.X == p2.X && a2.Y == p2.Y) ||
                    (a2.X == p1.X && a2.Y == p1.Y && a1.X == p2.X && a1.Y == p1.Y))
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Metoda overi, ci sa bod nachadza na niektorej alfa hrane
        /// </summary>
        /// <param name="p">hladany bod</param>
        /// <param name="pointsAll">mnozina vsetkych bodov - kedze alfa hrana obsahuje iba cislo bodu v mnozine, nie bod samotny</param>
        /// <param name="edges">alfa hrany</param>
        /// <returns>true - ak sa bod v hrane nachadza, nie ak sa nenachadza</returns>
        public static Boolean isPointInSomeEdge(CoridorPoint p, Stack<System.Drawing.Point> pointsAll, List<Diplo.Libs.Geometry.AlphaHull.Geometry.Edge> edges)
        {

            foreach (Diplo.Libs.Geometry.AlphaHull.Geometry.Edge edge in edges)
            {
                System.Drawing.Point p1 = pointsAll.ElementAt(edge.p1);
                System.Drawing.Point p2 = pointsAll.ElementAt(edge.p2);

                if (p1.X == p.X && p1.Y == p.Y) { return true; }
                if (p2.X == p.X && p2.Y == p.Y) { return true; }

            }

            return false;
        }

    }
}
