﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo;
using Diplo.MentalMap;
using Diplo.Libs.Geometry.AlphaHull.Geometry;

namespace Diplo.Libs.Geometry.AlphaHull
{
    /// <summary>
    /// Hlavná trieda pre počítanie ALFA HRÁN
    /// </summary>
    class AlphaHullObject
    {

        /// <summary>
        /// Z mnoziny bodov sa vypocitaju alfa hrany - podla parametru alfa / polomer kruznice
        /// </summary>
        /// <param name="CoridorMap"></param>
        /// <param name="g">graficky komponent</param>
        /// <param name="points">body</param>
        /// <param name="alpha">alfa parameter - polomer kruznice</param>
        /// <returns></returns>
        public static List<Diplo.Libs.Geometry.AlphaHull.Geometry.Edge> createAlphaHull(CoridorMap CoridorMap, 
            Graphics g, 
            Stack<System.Drawing.Point> points, 
            double alpha)
        {
            List<Diplo.Libs.Geometry.AlphaHull.Geometry.Edge> edges = new List<Diplo.Libs.Geometry.AlphaHull.Geometry.Edge>();

            // prekonvertujeme na body triangulatoru
            List<Diplo.Libs.Geometry.AlphaHull.Geometry.Point> Vertex = new List<Diplo.Libs.Geometry.AlphaHull.Geometry.Point>();
            foreach (System.Drawing.Point p in points)
            {
                Vertex.Add(new Diplo.Libs.Geometry.AlphaHull.Geometry.Point(p.X, p.Y));
            }

            // vytvorime triangulaciu
            List<Diplo.Libs.Geometry.AlphaHull.Geometry.Triangle> Triangle = Diplo.Libs.Geometry.AlphaHull.Geometry.Delauney.Triangulate(Vertex);
            
            // vykreslime trojuholniky
            foreach (Diplo.Libs.Geometry.AlphaHull.Geometry.Triangle tria in Triangle)
            {
                Diplo.Libs.Geometry.AlphaHull.Geometry.Point p1 = Vertex.ElementAt(tria.p1);
                Diplo.Libs.Geometry.AlphaHull.Geometry.Point p2 = Vertex.ElementAt(tria.p2);
                Diplo.Libs.Geometry.AlphaHull.Geometry.Point p3 = Vertex.ElementAt(tria.p3);

                if (CoridorMap.drawTriangulation)
                {
                    AlphaDrawHelper.drawTriangulation(g, p1, p2, p3);
                }
                
            }

            // vytvorime si voronoiove diagrami
            // resp. hladame vrcholy, kde sa pretinaju
            List<Diplo.Libs.Geometry.AlphaHull.Geometry.CircumCircle> centers = new List<Diplo.Libs.Geometry.AlphaHull.Geometry.CircumCircle>();
            foreach (Diplo.Libs.Geometry.AlphaHull.Geometry.Triangle tria in Triangle)
            {
                centers.Add(Voronoi.findCircumscribe(g, tria, Vertex));
               
            }

            //Console.WriteLine(alpha + "|" + (-1 / alpha));

            // rozhodnutie, ci hrana z triangulacie je alfa hrana
            AlphaShape alfaShape;
            foreach (Diplo.Libs.Geometry.AlphaHull.Geometry.Triangle tria in Triangle)
            {
                alfaShape = new AlphaShape(g, CoridorMap, alpha, Vertex, centers, tria.p1, tria.p2, tria);
                if (alfaShape.ret) 
                {
                    edges.Add(new Diplo.Libs.Geometry.AlphaHull.Geometry.Edge(tria.p1, tria.p2));

                    /*System.Drawing.Point p1 = new System.Drawing.Point(
                        Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p1).X)), 
                        Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p1).Y))
                        );
                    System.Drawing.Point p2 = new System.Drawing.Point(
                        Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p2).X)),
                        Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p2).Y))
                        );
                    g.DrawLine(System.Drawing.Pens.Green, p1, p2);*/
                }

                alfaShape = new AlphaShape(g, CoridorMap, alpha, Vertex, centers, tria.p2, tria.p3, tria);
                if (alfaShape.ret)
                {
                    edges.Add(new Diplo.Libs.Geometry.AlphaHull.Geometry.Edge(tria.p2, tria.p3));

                    /*System.Drawing.Point p1 = new System.Drawing.Point(
                       Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p2).X)),
                       Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p2).Y))
                       );
                    System.Drawing.Point p2 = new System.Drawing.Point(
                        Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p3).X)),
                        Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p3).Y))
                        );
                    g.DrawLine(System.Drawing.Pens.Green, p1, p2);*/
                }

                alfaShape = new AlphaShape(g, CoridorMap, alpha, Vertex, centers, tria.p3, tria.p1, tria);
                if (alfaShape.ret)
                {
                    edges.Add(new Diplo.Libs.Geometry.AlphaHull.Geometry.Edge(tria.p3, tria.p1));

                    /*System.Drawing.Point p1 = new System.Drawing.Point(
                       Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p3).X)),
                       Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p3).Y))
                       );
                    System.Drawing.Point p2 = new System.Drawing.Point(
                        Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p1).X)),
                        Convert.ToInt32(Math.Round(Vertex.ElementAt(tria.p1).Y))
                        );
                    g.DrawLine(System.Drawing.Pens.Green, p1, p2);*/
                }
            }

            return edges;

        }

        
    }
}
