﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.MentalMap;

namespace Diplo.Libs.Geometry.AlphaHull.Geometry
{
    class AlphaShape
    {
        // privatne premenne
        private int p1;
        private int p2;

        private Diplo.Libs.Geometry.AlphaHull.Geometry.Point voronoiVertice1 = null;
        private Diplo.Libs.Geometry.AlphaHull.Geometry.Point voronoiVertice2 = null;

        private double alfaMin;
        private double alfaMax;

        // flaf, ci mame alfa hranu
        public Boolean ret = false;

        public AlphaShape(Graphics g, CoridorMap CoridorMap, double alfa, List<Diplo.Libs.Geometry.AlphaHull.Geometry.Point> Vertex,
            List<Diplo.Libs.Geometry.AlphaHull.Geometry.CircumCircle> centers, int p1, int p2, Diplo.Libs.Geometry.AlphaHull.Geometry.Triangle triangle)
        {

            this.p1 = p1;
            this.p2 = p2;

            Diplo.Libs.Geometry.AlphaHull.Geometry.Point x = Vertex.ElementAt(p1);
            Diplo.Libs.Geometry.AlphaHull.Geometry.Point y = Vertex.ElementAt(p2);

            // najdeme take stredy, ktore su rovnako vzdialene od bodu p1 a p2
            foreach (Diplo.Libs.Geometry.AlphaHull.Geometry.CircumCircle center in centers)
            {
                // vzdialenost stredu od bodu p1
                double Xd1 = Math.Abs(center.center.X - x.X);
                double Xd2 = Math.Abs(center.center.Y - x.Y);
                double Xradius = Math.Sqrt(Xd1 * Xd1 + Xd2 * Xd2);
                // vzdialenost stredu od bodup2
                double Yd1 = Math.Abs(center.center.X - y.X);
                double Yd2 = Math.Abs(center.center.Y - y.Y);
                double Yradius = Math.Sqrt(Yd1 * Yd1 + Yd2 * Yd2);
                // ak sa rovnaju, mame bod

                if ((Math.Abs(Xradius - Yradius) <= 0.001) && isValid(Vertex, x, y, center))
                {

                    if (this.voronoiVertice1 == null)
                    {
                        this.voronoiVertice1 = new Diplo.Libs.Geometry.AlphaHull.Geometry.Point(center.center.X, center.center.Y);
                    }
                    else
                    {
                        this.voronoiVertice2 = new Diplo.Libs.Geometry.AlphaHull.Geometry.Point(center.center.X, center.center.Y);
                    }

                }

            }

            // spocitame alfaMin - najmensiu
            // ak mame dva body, je to priama vzdialenost medzi nimi
            if (this.voronoiVertice1 != null && this.voronoiVertice2 != null)
            {
                double d1 = Math.Abs(x.X - y.X);
                double d2 = Math.Abs(x.Y - y.Y);
                this.alfaMin = Math.Sqrt(d1 * d1 + d2 * d2);
            }
            // inac je to vzdialenost k ich najdenemu stredu (stred opisanej kruznice pre delaneu trojuholnik)
            if (this.voronoiVertice1 != null && this.voronoiVertice2 == null)
            {
                double d1 = Math.Abs(this.voronoiVertice1.X - x.X);
                double d2 = Math.Abs(this.voronoiVertice1.Y - x.Y);
                this.alfaMin = Math.Sqrt(d1 * d1 + d2 * d2);
            }

            // spocitame alfaMax
            // ak mame dva body, je to vzdialenost k najdenemu stredu
            if (this.voronoiVertice1 != null && this.voronoiVertice2 != null)
            {
                double d1 = Math.Abs(this.voronoiVertice1.X - x.X);
                double d2 = Math.Abs(this.voronoiVertice1.Y - x.Y);
                this.alfaMax = Math.Sqrt(d1 * d1 + d2 * d2);

                double d12 = Math.Abs(this.voronoiVertice2.X - x.X);
                double d22 = Math.Abs(this.voronoiVertice2.Y - x.Y);

                if (Math.Sqrt(d22 * d22 + d12 * d12) > this.alfaMax)
                {
                    this.alfaMax = Math.Sqrt(d22 * d22 + d12 * d12);
                }
            }
            // inac je to nekonecno
            if (this.voronoiVertice1 != null && this.voronoiVertice2 == null)
            {
                this.alfaMax = Double.PositiveInfinity;
            }
            
            //alfa = -1/alfa;
            alfa = alfa * -1;
            //Console.WriteLine(alfa);

            List<Diplo.Libs.Geometry.AlphaHull.Geometry.Point> Alfacenters = findCenterOfAlfaCircle(Vertex, triangle, this.alfaMin, this.alfaMax, p1, p2, 1 / alfa, this.voronoiVertice1 ,this.voronoiVertice2);


            foreach (Diplo.Libs.Geometry.AlphaHull.Geometry.Point center in Alfacenters)
            {
                if (center.X != -1 && center.Y != -1)
                {

                    if (CoridorMap.drawAlphacircles1)
                    {
                        g.DrawEllipse(System.Drawing.Pens.Orange, Convert.ToInt32(Math.Round(center.X - 1 / alfa / 2)), Convert.ToInt32(Math.Round(center.Y - 1 / alfa / 2)),
                            Convert.ToInt32(Math.Round(1 / alfa)), Convert.ToInt32(Math.Round(1 / alfa)));
                    }

                    if (CoridorMap.drawTriangulation1)
                    {
                        g.DrawLine(System.Drawing.Pens.Red, Convert.ToInt32(Math.Round(x.X)), Convert.ToInt32(Math.Round(x.Y)),
                            Convert.ToInt32(Math.Round(y.X)), Convert.ToInt32(Math.Round(y.Y)));
                    }

                    this.ret = true;
                }
            }



        }




        internal static List<Diplo.Libs.Geometry.AlphaHull.Geometry.Point> findCenterOfAlfaCircle(List<Diplo.Libs.Geometry.AlphaHull.Geometry.Point> Vertex, Diplo.Libs.Geometry.AlphaHull.Geometry.Triangle triangle,
            double alfaMin, double alfaMax,
            int pI1, int pI2, double r, Diplo.Libs.Geometry.AlphaHull.Geometry.Point voronoiVertice1, Diplo.Libs.Geometry.AlphaHull.Geometry.Point voronoiVertice2)
        {
            
            List<Diplo.Libs.Geometry.AlphaHull.Geometry.Point> circles = new List<Diplo.Libs.Geometry.AlphaHull.Geometry.Point>();


            Diplo.Libs.Geometry.AlphaHull.Geometry.Point p1 = Vertex.ElementAt(pI1);
            Diplo.Libs.Geometry.AlphaHull.Geometry.Point p2 = Vertex.ElementAt(pI2);

            double d1 = Math.Abs(p1.X - p2.X);
            double d2 = Math.Abs(p1.Y - p2.Y);
            double d;
            double q = Math.Sqrt(d1 * d1 + d2 * d2); // vzdialenost medzi d1 a d2

            double x3 = (p1.X + p2.X) / 2;
            double y3 = (p1.Y+p2.Y)/2;

            double x = x3 + Math.Sqrt(Math.Abs(r/2 * r/2 - (q / 2) * (q / 2))) * (p1.Y - p2.Y) / q;
            double y = y3 + Math.Sqrt(Math.Abs(r/2 * r/2 - (q / 2) * (q / 2))) * (p2.X - p1.X) / q;
            circles.Add(new Diplo.Libs.Geometry.AlphaHull.Geometry.Point(x, y));
            x = x3 - Math.Sqrt(Math.Abs(r / 2 * r / 2 - (q / 2) * (q / 2))) * (p1.Y - p2.Y) / q;
            y = y3 - Math.Sqrt(Math.Abs(r / 2 * r / 2 - (q / 2) * (q / 2))) * (p2.X - p1.X) / q;
            circles.Add(new Diplo.Libs.Geometry.AlphaHull.Geometry.Point(x, y));

            foreach(Diplo.Libs.Geometry.AlphaHull.Geometry.Point center in circles)
            {
                d1 = Math.Abs(p1.X - center.X);
                d2 = Math.Abs(p1.Y - center.Y);
                d = Math.Sqrt(d1 * d1 + d2 * d2);
                if (d > r / 2 + 0.01)
                {
                    center.X = -1;
                    center.Y = -1;
                }
            }
           

            // kontrola 2a)
            if (voronoiVertice1 != null && voronoiVertice2 == null)
            {
                // ak je to priamka, najblizsi mozny stred je bod z voronoiovho diagramu
                foreach (Diplo.Libs.Geometry.AlphaHull.Geometry.Point center in circles)
                {
                    if (center.X != -1 && center.Y != -1)
                    {
                        d1 = Math.Abs(p1.X - center.X);
                        d2 = Math.Abs(p1.Y - center.Y);
                        d = Math.Sqrt(d1 * d1 + d2 * d2);
                        if (alfaMin > d)
                        {
                            center.X = -1;
                            center.Y = -1;
                        }

                    }
                }

                // treti bod trojuholniku (ktoreho sucastou je alha hrana) nemoze lezat v trojuholniku
                // najprv najdeme treti bod
                Diplo.Libs.Geometry.AlphaHull.Geometry.Point notInCirclePoint = Vertex.ElementAt(triangle.p1);
                if ((notInCirclePoint.X == p1.X && notInCirclePoint.Y == p1.Y) || (notInCirclePoint.X == p2.X && notInCirclePoint.Y == p2.Y))
                {
                    notInCirclePoint = Vertex.ElementAt(triangle.p2);
                    if ((notInCirclePoint.X == p1.X && notInCirclePoint.Y == p1.Y) || (notInCirclePoint.X == p2.X && notInCirclePoint.Y == p2.Y))
                    {
                        notInCirclePoint = Vertex.ElementAt(triangle.p3);
                    }
                }

                // pre oba vrcholy overim, ze alfa kruznica, ktoru vytvaraju, neobsahuje treti bod
                // 
                foreach (Diplo.Libs.Geometry.AlphaHull.Geometry.Point center in circles)
                {
                    if (center.X != -1 && center.Y != -1)
                    {
                        if (Math.Pow(notInCirclePoint.X - center.X, 2) + Math.Pow(notInCirclePoint.Y - center.Y, 2) < r / 2 * r / 2)
                        {
                            center.X = -1;
                            center.Y = -1;
                        }
                    }
                }

            }
            else
            {
                // kontrola 2b
                // musime zaistit, aby stred alfa kruznice lezal na usecke (medzi dvoma voronoiovimi kruznicami)
                foreach (Diplo.Libs.Geometry.AlphaHull.Geometry.Point center in circles)
                {
                    if (center.X != -1 && center.Y != -1)
                    {
                        if (voronoiVertice1.X != voronoiVertice2.X)
                        {
                            if ((voronoiVertice1.X <= center.X && center.X <= voronoiVertice2.X) || (voronoiVertice2.X <= center.X && center.X <= voronoiVertice1.X))
                            {

                            }
                            else
                            {
                                center.X = -1;
                                center.Y = -1;
                            }
                        }
                        else
                        {
                            if ((voronoiVertice1.Y <= center.Y && center.Y <= voronoiVertice2.Y) || (voronoiVertice2.Y <= center.Y && center.Y <= voronoiVertice1.Y))
                            {

                            }
                            else
                            {
                                center.X = -1;
                                center.Y = -1;
                            }

                        }

                        


                    }
                }
            }


            return circles;
        }




        internal static bool isValid(List<Diplo.Libs.Geometry.AlphaHull.Geometry.Point> Vertex, Diplo.Libs.Geometry.AlphaHull.Geometry.Point x, Diplo.Libs.Geometry.AlphaHull.Geometry.Point y,
            Diplo.Libs.Geometry.AlphaHull.Geometry.CircumCircle center)
        {
            Diplo.Libs.Geometry.AlphaHull.Geometry.Point c1 = Vertex.ElementAt(center.p1);
            Diplo.Libs.Geometry.AlphaHull.Geometry.Point c2 = Vertex.ElementAt(center.p2);
            Diplo.Libs.Geometry.AlphaHull.Geometry.Point c3 = Vertex.ElementAt(center.p3);

            int common = 0;
            if ((c1.X == x.X && c1.Y == x.Y) || (c2.X == x.X && c2.Y == x.Y) || (c3.X == x.X && c3.Y == x.Y))
            {
                common++;
            }

            if ((c1.X == y.X && c1.Y == y.Y) || (c2.X == y.X && c2.Y == y.Y) || (c3.X == y.X && c3.Y == y.Y))
            {
                common++;
            }
            
            if(common >= 2){ return true; }
            return false;
        }

    }
}
