﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using Diplo.Libs.Geometry.ConcaveObjects;
using Diplo.Libs.Drawing;

namespace Diplo.Libs.Geometry
{
    /// <summary>
    /// Počítanie konkávneho obalu pomocou metód podobných výpočtu konvexného obalu.
    /// </summary>
    class ConcaveHull
    {
        /// <summary>
        /// <para>Výpočet konkávneho obalu pomocou digging operácie</para>
        /// </summary>
        /// <param name="g">Grafický komponent.</param>
        /// <param name="points">Body, pre ktoré sa počíta konkávny obal.</param>
        /// <returns>Množina bodov, ktoré vytvárajú konkávny obal - mnohouholník.</returns>
        static public Stack<Point> computeConcaveHull(Graphics g, Stack<Point> points)
        {

            g.FillEllipse(new SolidBrush(Color.Pink), 213, 213 , 14, 14);
            points.Push(new Point(220, 220));

            // preprocesing
            Stack<Point> convexList = ConvexHull.computeConvexHull(g, points);
            int N = 3; // threshold
            Stack<Point> concaveList = ConvexHull.computeConvexHull(g, points);



            // digging
            for (int i = 0; i < concaveList.Count()-1; i++)
            {
                Console.WriteLine(i);
                Point ci1 = concaveList.ElementAt(i);
                Point ci2 = concaveList.ElementAt(i + 1);

                int min = 999999;
                int minId = -1;
                // find the nearest inner point pk ∈ G from the edge (ci1,ci2);
                for (int j = 0; j < points.Count(); j++)
                {
                    if (!StackHelper.isPointInStack(points.ElementAt(j), convexList) && !StackHelper.isPointInStack(points.ElementAt(j), concaveList)) // nie je v convexlisrte - je to vnutorny bod
                    {
                        int dist = DE(points.ElementAt(j), ci1, ci2);
                        if (dist < min)
                        {
                            minId = j;
                            min = dist;
                        }
                    }
                }


                if (minId > -1)
                {
                    //Point pk = points.ElementAt(minId);
                    Point pk = new Point();
                    pk.X = points.ElementAt(minId).X;
                    pk.Y = points.ElementAt(minId).Y;

                    int eh = TwoPoints.manhatan_distance(ci1,ci2); // the length of the edge
                    
                    Stack<Point> ddStack = new Stack<Point>();
                    ddStack.Push(ci1);
                    ddStack.Push(ci2);
                    int dd = DD(pk, ddStack);

                    if (dd > 0)
                    {
                        if ((eh / dd) > N) // digging process
                        {
                            Stack<Point> newConcave = new Stack<Point>();
                            newConcave.Push(ci2);
                            newConcave.Push(pk);
                            newConcave.Push(ci1);

                            for (int j = concaveList.Count() - 1; j >= 0; j--)
                            {
                                Boolean add = true;
                                if (concaveList.ElementAt(j).X == ci1.X && concaveList.ElementAt(j).Y == ci1.Y)
                                {
                                    add = false;
                                }

                                if (concaveList.ElementAt(j).X == ci2.X && concaveList.ElementAt(j).Y == ci2.Y)
                                {
                                    add = false;
                                }

                                if (add)
                                {
                                    newConcave.Push(concaveList.ElementAt(j));
                                }
                            }

                            concaveList.Clear();
                            StackHelper.copyStackToStack(newConcave, concaveList);
                        }
                    }

                    

                }

            }

            /*for (int i = 0; i < concaveList.Count()-1; i++)
            {
                g.DrawLine(System.Drawing.Pens.Blue, concaveList.ElementAt(i).X, concaveList.ElementAt(i).Y, concaveList.ElementAt(i+1).X, concaveList.ElementAt(i+1).Y);
                g.FillEllipse(new SolidBrush(Color.Blue), concaveList.ElementAt(i).X - 3, concaveList.ElementAt(i).Y - 3, 6, 6);
            }*/

            return concaveList;
        }

        /// <summary>
        /// <para>Výpočet konkávneho obalu pomocou digging operácie (alternatívna implementácia)</para>
        /// </summary>
        /// <param name="g">Grafický komponent.</param>
        /// <param name="points">Body, pre ktoré sa počíta konkávny obal.</param>
        /// <returns>Množina bodov, ktoré vytvárajú konkávny obal - mnohouholník.</returns>
        static public Stack<Point> computeConcaveHull2(Graphics g, Stack<Point> points)
        {
            // preprocesing
            Stack<Point> convexList = ConvexHull.computeConvexHull(g, points);
            // prerobime na stack hran
            List<HullEdge> convexEdgeList = new List<HullEdge>();
            for (int i = 0; i < convexList.Count()-1;i++ )
            {
                convexEdgeList.Add(new HullEdge(convexList.ElementAt(i), convexList.ElementAt(i+1)));
            }

            double N = 1.5; // threshold

            List<HullEdge> concavEdgeList = new List<HullEdge>();
            for (int i = 0; i < convexList.Count() - 1; i++)
            {
                concavEdgeList.Add(new HullEdge(convexList.ElementAt(i), convexList.ElementAt(i + 1)));
            }

            // digging
            for (int i = 0; i < concavEdgeList.Count() - 1; i++)
            {
                HullEdge edge = concavEdgeList.ElementAt(i);

                int min = 999999;
                int minId = -1;
                // find the nearest inner point pk ∈ G from the edge (ci1,ci2);
                for (int j = 0; j < points.Count(); j++)
                {
                    if (!StackHelper.isPointInStack(points.ElementAt(j), convexList) && !isPointInEdgeStack(points.ElementAt(j), convexEdgeList)) // nie je v convexlisrte - je to vnutorny bod
                    {
                        int dist = DE(points.ElementAt(j), edge.x, edge.y);
                        if (dist < min)
                        {
                            minId = j;
                            min = dist;
                        }
                    }
                }


                if (minId > -1)
                {
                    Point pk = new Point();
                    pk.X = points.ElementAt(minId).X;
                    pk.Y = points.ElementAt(minId).Y;

                    int eh = TwoPoints.manhatan_distance(edge.x, edge.y); // the length of the edge

                    Stack<Point> ddStack = new Stack<Point>();
                    ddStack.Push(edge.x);
                    ddStack.Push(edge.y);
                    int dd = DD(pk, ddStack);

                    if (dd > 0)
                    {
                        if ((eh / dd) > N) // digging process
                        {
                            concavEdgeList.Add(new HullEdge(edge.x, pk));
                            concavEdgeList.Add(new HullEdge(pk, edge.y));

                            for (int j = 0; j < concavEdgeList.Count(); j++)
                            {
                                HullEdge oldEdge = concavEdgeList.ElementAt(j);

                                if (oldEdge.x.X == edge.x.X && oldEdge.x.Y == edge.x.Y && oldEdge.y.X == edge.y.X && oldEdge.y.Y == edge.y.Y)
                                {
                                    concavEdgeList.RemoveAt(j);
                                }

                            }

                            i = 0;
                        }
                    }
                }
            }

            for (int i = 0; i < concavEdgeList.Count(); i++)
            {
                g.DrawLine(System.Drawing.Pens.Blue, concavEdgeList.ElementAt(i).x.X, concavEdgeList.ElementAt(i).x.Y, concavEdgeList.ElementAt(i).y.X, concavEdgeList.ElementAt(i).y.Y);
                g.FillEllipse(new SolidBrush(Color.Blue), concavEdgeList.ElementAt(i).x.X - 3, concavEdgeList.ElementAt(i).x.Y - 3, 6, 6);
            }

            return convexList;
        }

        
        static public Boolean isPointInEdgeStack(Point p, List<HullEdge> s)
        {
            foreach (HullEdge v in s)
            {
                if (v.x.X == p.X && v.x.Y == p.Y) { return true; }
                if (v.y.X == p.X && v.y.Y == p.Y) { return true; }
            }
            return false;
        }

        /// <summary>
        /// <para>
        /// minimalna vzdialenost medzi bodom a mnozinou bodoov
        /// DD(p ,q) = min{D(p, q1), D(p, q2), , D(p, q3),.. , D(p, qn)}
        /// </para>
        /// </summary>
        /// <param name="p"></param>
        /// <param name="q"></param>
        /// <returns></returns>
        static public int DD(Point p, Stack<Point> q)
        {
            int dist = 999999;

            foreach(Point v in q){
                int d = TwoPoints.manhatan_distance(p, v);
                if (d < dist) { dist = d; }
            }

            return dist;
        }

        /// <summary>
        /// minimalna vzdialenost medzi bodom a useckou (e1, e2)
        /// DE(p, (e1,e2)) = min{ ek ∈ (e1,e2) | D(p, ek)}
        /// </summary>
        /// <param name="p"></param>
        /// <param name="e1"></param>
        /// <param name="e2"></param>
        /// <returns></returns>
        static public int DE(Point p, Point e1, Point e2)
        {
            int dist = TwoPoints.manhatan_distance(p, e1);
            int dist2 = TwoPoints.manhatan_distance(p, e2);
            if (dist < dist2) { return dist; }
            else { return dist2; }
        }

        /// <summary>
        /// minimalna vzdialenost medzi bodom a trojuholnikom (t1, t2, t3)
        /// DT(p, (t1,t2,t3)) = min{ tk ∈ (t1,t2,t3) | D(p, tk)}
        /// </summary>
        /// <param name="p"></param>
        /// <param name="t1"></param>
        /// <param name="t2"></param>
        /// <param name="t3"></param>
        /// <returns></returns>
        static public int DT(Point p, Point t1, Point t2, Point t3)
        {
            int dist = TwoPoints.manhatan_distance(p, t1);
            int dist2 = TwoPoints.manhatan_distance(p, t2);
            int dist3 = TwoPoints.manhatan_distance(p, t3);

            int min = dist;
            if (dist2 < min) { min = dist2; }
            if (dist3 < min) { min = dist3; }

            return min;
        }

    }

    

}
