﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.Libs.Drawing;

namespace Diplo.Libs.Geometry
{   
    /// <summary>
    /// Výpočet konvexného obalu.
    /// </summary>
    class ConvexHull
    {

        /// <summary>
        /// Algoritmus ("gift-wraping") na vypocet convexneho obalu
        /// </summary>
        /// <param name="g">Graficky komponent</param>
        /// <param name="points">Body, z ktorych sa ma obal vypocitat</param>
        /// <returns>Body tvoriace konvexný obal</returns>
        static public Stack<Point> computeConvexHull(Graphics g, Stack<Point> points)
        {
            int minX = 9999999;
            int minId = -1;
            for (int i = 0; i < points.Count - 1; i++)
            {
                if (minX > points.ElementAt(i).X)
                {
                    minX = points.ElementAt(i).X;
                    minId = i;
                }
            }
            Point pointOnHull = new Point();
            pointOnHull.X = points.ElementAt(minId).X;
            pointOnHull.Y = points.ElementAt(minId).Y;

            Point endpoint = new Point();
            Stack<Point> P = new Stack<Point>();

            Boolean cont = true;

            while (cont)
            {

                P.Push(pointOnHull);

                endpoint.X = points.ElementAt(0).X;
                endpoint.Y = points.ElementAt(0).Y;

                for (int k = 0; k < points.Count; k++)
                {
                    if ((endpoint.X == pointOnHull.X && pointOnHull.Y == endpoint.Y)
                        || (PointPosition.isOnTheLeft(points.ElementAt(k).X, points.ElementAt(k).Y, P.ElementAt(0), endpoint)))
                    {
                        endpoint.X = points.ElementAt(k).X;
                        endpoint.Y = points.ElementAt(k).Y;
                    }

                }

                pointOnHull.X = endpoint.X;
                pointOnHull.Y = endpoint.Y;

                if (endpoint.X == P.ElementAt(P.Count - 1).X && endpoint.Y == P.ElementAt(P.Count - 1).Y)
                {
                    cont = false;
                }
            }
            return P;
        }

    }
}
