﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.MentalMap;
using Diplo.Libs.Drawing;
using Diplo.MentalMap.CoridorObjects;
using Diplo.GraphObjects;
using Diplo.Configs;

namespace Diplo.Libs.Geometry
{
    /// <summary>
    /// Pomocna trieda pre pracu s EdgeBox-om
    /// </summary>
    class EdgeBoxHelper
    {
        
        /// <summary>
        /// <para>*** GEOMETRICKA POMOCNA METODA *** </para>
        /// <para>Z vrcholu - grafu - vytvori pre dany vrchol EdgeBox. </para>
        /// <para>Vrati 4 body, ktore tvoria rohy EdgeBoxu - stvorca - okolo daneho vrcholu.</para>
        /// </summary>
        /// <param name="v"></param>
        /// <returns></returns>
        internal static Stack<Point> getBoxForVertice(Vertex v)
        {
            Stack<Point> ret = new Stack<Point>();

            Point R1 = new Point();
            Point R2 = new Point();
            Point R3 = new Point();
            Point R4 = new Point();

            R1.X = Convert.ToInt32(Math.Round(v.pos[0])) - Config.vertexArea;
            R1.Y = Convert.ToInt32(Math.Round(v.pos[1])) - Config.vertexArea;

            R2.X = Convert.ToInt32(Math.Round(v.pos[0])) + v.size + Config.vertexArea;
            R2.Y = Convert.ToInt32(Math.Round(v.pos[1])) - Config.vertexArea;

            R3.X = Convert.ToInt32(Math.Round(v.pos[0])) + v.size + Config.vertexArea;
            R3.Y = Convert.ToInt32(Math.Round(v.pos[1])) + v.size + Config.vertexArea;

            R4.X = Convert.ToInt32(Math.Round(v.pos[0])) - Config.vertexArea;
            R4.Y = Convert.ToInt32(Math.Round(v.pos[1])) + v.size + Config.vertexArea;

            ret.Push(R1);
            ret.Push(R2);
            ret.Push(R3);
            ret.Push(R4);

            return ret;
        }

        

        /// <summary>
        /// <para>*** GEOMETRICKA POMOCNA METODA *** </para>
        /// <para>Metoda pre dany VertixeBoxu a EdgeBox skontroluje, ci existuje nejaky bod leziaci na hranach VerticeBox-u,
        /// ktory lezi aj v edgeBoxu. </para>
        /// <para>Zisti, ci VerticeBox a EdgeBox su kolidujuce.</para>
        /// </summary>
        /// <param name="verticeBox"></param>
        /// <param name="edgeBox"></param>
        /// <returns></returns>
        internal static int checkEdgeBox(Stack<Point> verticeBox, Stack<Point> edgeBox)
        {
            // horna hrana
            for (int k = verticeBox.ElementAt(3).X; k < verticeBox.ElementAt(2).X; k = k + 5)
            {
                if (CoridorExtenderHelper.isPointInEdgeBox(new Point(k, verticeBox.ElementAt(3).Y), edgeBox))
                {
                    return 1;
                }
            }

            // prava hrana
            for (int k = verticeBox.ElementAt(2).Y; k < verticeBox.ElementAt(1).Y; k = k + 5)
            {
                if (CoridorExtenderHelper.isPointInEdgeBox(new Point(verticeBox.ElementAt(2).X, k), edgeBox))
                {
                    return 2;
                }
            }

            // dolna hrana
            for (int k = verticeBox.ElementAt(1).X; k > verticeBox.ElementAt(0).X; k = k - 5)
            {
                if (CoridorExtenderHelper.isPointInEdgeBox(new Point(k, verticeBox.ElementAt(1).Y), edgeBox))
                {
                    return 3;
                }
            }

            // lava hrana
            for (int k = verticeBox.ElementAt(0).Y; k > verticeBox.ElementAt(3).Y; k = k - 5)
            {
                if (CoridorExtenderHelper.isPointInEdgeBox(new Point(verticeBox.ElementAt(0).X, k), edgeBox))
                {
                    return 4;
                }
            }

            return 0;
        }

        /// <summary>
        /// <para>*** KRESLIACA POMOCNA METODA *** </para>
        /// <para>Vykresli - vybodkuje - tie body z VerticeBoxu (berie body na hranach VerticeBox-u),
        /// ktore lezia v EdgeBox-e (ktory je tiez dany ako parameter). </para>
        /// <para>Metoda vlastne najde spolocne body VerticeBox-u a EdgeBox-u.</para>
        /// </summary>
        /// <param name="g"></param>
        /// <param name="verticeBox"></param>
        /// <param name="edgeBox"></param>
        public static void drawVertexColiding(Graphics g, Stack<Point> verticeBox, Stack<Point> edgeBox)
        {
            // horna hrana
            for (int k = verticeBox.ElementAt(3).X; k < verticeBox.ElementAt(2).X; k = k + 5)
            {
                if (CoridorExtenderHelper.isPointInEdgeBox(new Point(k, verticeBox.ElementAt(3).Y), edgeBox))
                {
                    DrawPoint.drawPoint(g, new Point(k, verticeBox.ElementAt(3).Y));
                }
            }

            // prava hrana
            for (int k = verticeBox.ElementAt(2).Y; k < verticeBox.ElementAt(1).Y; k = k + 5)
            {
                if (CoridorExtenderHelper.isPointInEdgeBox(new Point(verticeBox.ElementAt(2).X, k), edgeBox))
                {
                    DrawPoint.drawPoint(g, new Point(verticeBox.ElementAt(2).X, k));
                }
            }

            // dolna hrana
            for (int k = verticeBox.ElementAt(1).X; k > verticeBox.ElementAt(0).X; k = k - 5)
            {
                if (CoridorExtenderHelper.isPointInEdgeBox(new Point(k, verticeBox.ElementAt(1).Y), edgeBox))
                {
                    DrawPoint.drawPoint(g, new Point(k, verticeBox.ElementAt(1).Y));
                }
            }

            // lava hrana
            for (int k = verticeBox.ElementAt(0).Y; k > verticeBox.ElementAt(3).Y; k = k - 5)
            {
                if (CoridorExtenderHelper.isPointInEdgeBox(new Point(verticeBox.ElementAt(0).X, k), edgeBox))
                {
                    DrawPoint.drawPoint(g, new Point(verticeBox.ElementAt(0).X, k));
                }
            }

        }
    }
}
