﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Diplo.Libs.Geometry
{
    /// <summary>
    /// Trieda poskytujúca základne operácie s úsečkou / cestou
    /// </summary>
    class LineSegment
    {

        /// <summary>
        /// <para>Vypocita dlzku cesty</para>
        /// </summary>
        /// <param name="path">Cesta tvorena bodmi (System.Drawing.Point)</param>
        /// <returns></returns>
        public static double pathLength(Stack<Point> path)
        {
            double length = 0;

            for (int i = 1; i < path.Count(); i++)
            {
                Point p1 = path.ElementAt(i - 1);
                Point p2 = path.ElementAt(i);

                double dx = Math.Pow(Math.Abs(p1.X - p2.X), 2);
                double dy = Math.Pow(Math.Abs(p1.Y - p2.Y), 2);
                length += Math.Floor(Math.Sqrt(dx + dy));
            }

            return length;
        }

        /// <summary>
        /// <para>Medzi dvoma bodmi - na usecke - najde bod, ktory sa sa nachadza urcitu vzdialenost (danu ako parameter) od bodu p1</para>
        /// </summary>
        /// <param name="p1">Bod 1 usecky</param>
        /// <param name="p2">Bod 2 usecky</param>
        /// <param name="distance">vzdialenost od bodu usecky, v ktorej bude bod</param>
        /// <returns></returns>
        public static Point middlePoint(Point p1, Point p2, double distance)
        {
            Point middlePoint = new Point();

            // vypocitame vzdialenost medzi p1 a p2
            double dx = Math.Pow(Math.Abs(p1.X - p2.X), 2);
            double dy = Math.Pow(Math.Abs(p1.Y - p2.Y), 2);
            double length = Math.Floor(Math.Sqrt(dx + dy));

            double multi = length / distance;

            // vypocitame y-suradnicu
            double yCoordination = (Math.Abs(p1.Y - p2.Y) / multi);
            if (p1.Y > p2.Y) { yCoordination = Math.Floor(p1.Y - yCoordination); }
            else { yCoordination = Math.Floor(p1.Y + yCoordination); }

            // vypocitame x-suradnicu
            double xCoordination = (Math.Abs(p1.X - p2.X) / multi);
            if (p1.X > p2.X) { xCoordination = Math.Floor(p1.X - xCoordination); }
            else { xCoordination = Math.Floor(p1.X + xCoordination); }

            middlePoint.X = Convert.ToInt32(xCoordination);
            middlePoint.Y = Convert.ToInt32(yCoordination);

            return middlePoint;
        }


        /// <summary>
        /// <para>Do line segments (x1, y1)--(x2, y2) and (x3, y3)--(x4, y4) intersect?</para>
        /// </summary>
        /// <param name="x1"></param>
        /// <param name="y1"></param>
        /// <param name="x2"></param>
        /// <param name="y2"></param>
        /// <param name="x3"></param>
        /// <param name="y3"></param>
        /// <param name="x4"></param>
        /// <param name="y4"></param>
        /// <returns></returns>
        public static bool DoLineSegmentsIntersect(double x1, double y1, double x2, double y2, double x3, double y3, double x4, double y4)
        {
            int d1 = ComputeDirection(x3, y3, x4, y4, x1, y1);
            int d2 = ComputeDirection(x3, y3, x4, y4, x2, y2);
            int d3 = ComputeDirection(x1, y1, x2, y2, x3, y3);
            int d4 = ComputeDirection(x1, y1, x2, y2, x4, y4);
            return (((d1 > 0 && d2 < 0) || (d1 < 0 && d2 > 0)) &&
                    ((d3 > 0 && d4 < 0) || (d3 < 0 && d4 > 0))) ||
                   (d1 == 0 && IsOnSegment(x3, y3, x4, y4, x1, y1)) ||
                   (d2 == 0 && IsOnSegment(x3, y3, x4, y4, x2, y2)) ||
                   (d3 == 0 && IsOnSegment(x1, y1, x2, y2, x3, y3)) ||
                   (d4 == 0 && IsOnSegment(x1, y1, x2, y2, x4, y4));
        }

        /// <summary>
        /// <para></para>
        /// </summary>
        /// <param name="xi"></param>
        /// <param name="yi"></param>
        /// <param name="xj"></param>
        /// <param name="yj"></param>
        /// <param name="xk"></param>
        /// <param name="yk"></param>
        /// <returns></returns>
        public static bool IsOnSegment(double xi, double yi, double xj, double yj,
                        double xk, double yk)
        {
            return (xi <= xk || xj <= xk) && (xk <= xi || xk <= xj) &&
                   (yi <= yk || yj <= yk) && (yk <= yi || yk <= yj);
        }

        /// <summary>
        /// <para></para>
        /// </summary>
        /// <param name="xi"></param>
        /// <param name="yi"></param>
        /// <param name="xj"></param>
        /// <param name="yj"></param>
        /// <param name="xk"></param>
        /// <param name="yk"></param>
        /// <returns></returns>
        public static int ComputeDirection(double xi, double yi, double xj, double yj, double xk, double yk)
        {
            double a = (xk - xi) * (yj - yi);
            double b = (xj - xi) * (yk - yi);
            return a < b ? -1 : a > b ? 1 : 0;
        }

    }
}
