﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Diplo.Libs.Geometry
{
    /// <summary>
    /// Metoda posudzuje, či bod je vo vnútri mnohouholníka - poligónu.
    /// Overovanie sa zaistí tak, že sa overí, či bod leži "na ľavo" od každej hrany mnohouholníka.
    /// </summary>
    class PointPosition
    {
        /// <summary>
        /// <para>Metóda určí, či bod leží v mnohouholníku - poligónu.</para>
        /// </summary>
        /// <param name="P">Bod, ktorý sa posudzuje či je v strede poligónu.</param>
        /// <param name="edgeBox">Poligón, ktorý je charakterizovaný množinou bodov.</param>
        /// <returns></returns>
        public static Boolean isPointInPoligon(Point P, Stack<Point> edgeBox)
        {
            Boolean allOnLeft = true;
            // zoberieme jednu z 4 hran edgebox a bod
            for (int l = 0; l < edgeBox.Count - 1; l++)
            {
                if (!isOnTheLeft(P.X, P.Y, edgeBox.ElementAt(l), edgeBox.ElementAt(l + 1)))
                {
                    allOnLeft = false;
                }
            }

            if (edgeBox.Count > 0)
            {
                if (!isOnTheLeft(P.X, P.Y, edgeBox.ElementAt(edgeBox.Count - 1), edgeBox.ElementAt(0)))
                {
                    allOnLeft = false;
                }
            }

            return allOnLeft;
        }

        /// <summary>
        /// <para>Určenie, či bod leží na ľavo od úsečky.</para>
        /// </summary>
        /// <param name="xp">X parameter bodu pre overenie</param>
        /// <param name="yp">Y parameter bodu pre overenie</param>
        /// <param name="p1">Bod 1 úsečky</param>
        /// <param name="p2">Bod 2 úsečky</param>
        /// <returns></returns>
        public static Boolean isOnTheLeft(int xp, int yp, Point p1, Point p2)
        {
            float A = -(p2.Y - p1.Y);
            float B = p2.X - p1.X;
            float C = -(A * p1.X + B * p1.Y);

            float D = A * xp + B * yp + C;

            if (D < 0) { return true; }
            return false;
        }

    }
}
