﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.Libs.Drawing;

namespace Diplo.Libs.Geometry.Smart
{
    /// <summary>
    /// Reprezentacia usecky pomocou pokrocileho objektu (usecka ma pridane dalsie uzitocne metody).
    /// </summary>
    class SmartLineSegment
    {
        public Point p1;
        public Point p2;

        public double a;
        public double b;

        /// <summary>
        /// <para>Vytvori objekt LINE SEGMENT
        /// V konstrukture sa vytvori aj rovnica usecky - resp. priamky</para>
        /// </summary>
        /// <param name="P1"></param>
        /// <param name="P2"></param>
        public SmartLineSegment(Point P1, Point P2)
        {
            this.p1 = P1;
            this.p2 = P2;

            // vypocet rovnice
            // a :
            this.a = (Convert.ToDouble(P1.Y) - Convert.ToDouble(P2.Y)) / (Convert.ToDouble(P1.X) - Convert.ToDouble(P2.X));
            // b :
            this.b = Convert.ToDouble(P1.Y) - this.a * Convert.ToDouble(P1.X);
        }

        /// <summary>
        /// <para>Na vstupe je index jedneho z bodu usecky a vzdialenost. Vrati bod, ktori 
        /// - je od bodu (urceneho indexom) vzdialeny o urcenu vzdialenost
        /// - lezi na priamke, na ktorej aj usecka
        /// - vrateni bod na usecke nelezi</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="pointIndex">Index jedneho z krajných bodov (možnosti sú 1 a 2)</param>
        /// <param name="distance">Vzdialenosť nového bodu od krajného bodu.</param>
        /// <returns>Výsledný bod</returns>
        public Point getPointOutOfLine(Graphics g, int pointIndex, int distance)
        {
            Point P = new Point();
            if (pointIndex == 1 || pointIndex == 2)
            {
                Point MainPoint;
                Point SecPoint;
                if (pointIndex == 1)
                {
                    MainPoint = this.p1;
                    SecPoint = this.p2;
                }
                else
                {
                    MainPoint = this.p2;
                    SecPoint = this.p1;
                }

                Point inter1 = new Point();
                Point inter2 = new Point();

                Point leftMax = new Point();
                Point rightMax = new Point();

                if (Double.IsNegativeInfinity(this.b) || Double.IsPositiveInfinity(this.b))
                {
                    if (Double.IsNegativeInfinity(this.b))
                    {
                        rightMax = new Point(this.p1.X, 0);
                        leftMax = new Point(this.p1.X, 1000);
                    }
                    else
                    {
                        leftMax = new Point(this.p1.X, 0);
                        rightMax = new Point(this.p1.X, 1000);
                    }

                }
                else
                {

                    leftMax = new Point(0, Convert.ToInt32(Math.Round(this.b)));
                    rightMax = new Point(1000, Convert.ToInt32(Math.Round(1000 * this.a + this.b)));
                }

                this.FindLineCircleIntersections(MainPoint, distance, leftMax, rightMax, out inter1, out inter2);

                if (TwoPoints.euclidDistance(SecPoint, inter1) > TwoPoints.euclidDistance(SecPoint, inter2))
                {
                    return inter1;
                }
                else
                {
                    return inter2;
                }


            }
            return P;
        }

        /// <summary>
        /// <para>Rozhodne ci bod lezi na usecke
        /// TODO : treba overit ci sedi aj rovnici priamke</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="P">Bod, ktorý sa bdue určovať</param>
        /// <returns></returns>
        public Boolean isPointOnLine(Graphics g, Point P)
        {
            Boolean isOnLine = true;
            if (this.p1.X < this.p2.X)
            {
                
                if (this.p1.X < P.X && P.X < this.p2.X)
                {
                    
                }
                else
                {
                    isOnLine = false;
                }
            }
            else
            {
                if (this.p2.X < P.X && P.X < this.p1.X)
                {

                }
                else
                {
                    isOnLine = false;
                }
            }

            if (this.p1.Y < this.p2.Y)
            {
                if (this.p2.Y < P.Y && P.Y < this.p1.Y)
                {

                }
                else
                {
                    isOnLine = false;
                }
            }
            else
            {
                if (this.p1.Y > P.Y && P.Y > this.p2.Y)
                {

                }
                else
                {
                    isOnLine = false;
                }
            }

            return isOnLine;
        }

        /// <summary>
        ///<para>Vykresli kusok usecky</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        public void _drawLine(Graphics g)
        {
            for (int i = 0; i < 20; i++)
            {
                DrawPoint.drawHighligthedPoint(g, new Point((i * 10), 
                    Convert.ToInt32(Math.Round(i * 10 * this.a + this.b))), "Green");
            }

        }

        /// <summary>
        /// <para></para>
        /// </summary>
        /// <param name="middlePoint"></param>
        /// <param name="radius"></param>
        /// <param name="point1"></param>
        /// <param name="point2"></param>
        /// <param name="intersection1"></param>
        /// <param name="intersection2"></param>
        /// <returns></returns>
        private int FindLineCircleIntersections(Point middlePoint, float radius, Point point1, Point point2, out Point intersection1, out Point intersection2)
        {
            float dx, dy, A, B, C, det, t;

            dx = point2.X - point1.X;
            dy = point2.Y - point1.Y;

            A = dx * dx + dy * dy;
            B = 2 * (dx * (point1.X - middlePoint.X) + dy * (point1.Y - middlePoint.Y));
            C = (point1.X - middlePoint.X) * (point1.X - middlePoint.X) + (point1.Y - middlePoint.Y) * (point1.Y - middlePoint.Y) - radius * radius;

            det = B * B - 4 * A * C;
            if ((A <= 0.0000001) || (det < 0))
            {
                // No real solutions.
                intersection1 = new Point(0, 0);
                intersection2 = new Point(0, 0); ;
                return 0;
            }
            else if (det == 0)
            {
                // One solution.
                t = -B / (2 * A);
                intersection1 = new Point(Convert.ToInt32(Math.Round(point1.X + t * dx)), Convert.ToInt32(Math.Round(point1.Y + t * dy)));
                intersection2 = new Point(0, 0);
                return 1;
            }
            else
            {
                // Two solutions.
                t = (float)((-B + Math.Sqrt(det)) / (2 * A));
                intersection1 = new Point(Convert.ToInt32(Math.Round(point1.X + t * dx)), Convert.ToInt32(Math.Round(point1.Y + t * dy)));
                t = (float)((-B - Math.Sqrt(det)) / (2 * A));
                intersection2 = new Point(Convert.ToInt32(Math.Round(point1.X + t * dx)), Convert.ToInt32(Math.Round(point1.Y + t * dy)));
                return 2;
            }
        }

    }
}
