﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.MentalMap;
using Diplo.MentalMap.CoridorObjects;

namespace Diplo.Libs.Geometry
{
    /// <summary>
    /// Knižnica pre meranie vzdialenosti medzi dvoma bodmi. Knižnica poskytuje meranie 
    /// vzdialenosti pomocou euklidovskej vzdialenosti a tkz. manhatanovej vzdialenosti.
    /// </summary>
    class TwoPoints
    {
        /// <summary>
        /// <para>Určí manhatanovsku vzdialenosť medzi dvoma bodmi.</para>
        /// </summary>
        /// <param name="start">Bod X</param>
        /// <param name="end">Bod Y</param>
        /// <returns></returns>
        static public int manhatan_distance(Point start, Point end)
        {
            return Math.Abs(start.X - end.X) + Math.Abs(start.Y - end.Y);
        }

        /// <summary>
        /// <para>Určí manhatanovsku vzdialenosť medzi dvoma bodmi.</para>
        /// </summary>
        /// <param name="start">Bod X</param>
        /// <param name="end">Bod Y</param>
        /// <returns></returns>
        static public int manhatan_distance(CoridorPoint start, CoridorPoint end)
        {
            return Math.Abs(start.X - end.X) + Math.Abs(start.Y - end.Y);
        }

        /// <summary>
        /// <para>Určí vzdialenosť medzi dvoma bodmi pomocou euklidovskej geomterie.</para>
        /// </summary>
        /// <param name="p1">Bod X</param>
        /// <param name="p2">Bod Y</param>
        /// <returns></returns>
        static public double euclidDistance(Point p1, Point p2)
        {
            return Math.Sqrt(Math.Pow(Convert.ToDouble(p2.X - p1.X), 2) 
                + Math.Pow(Convert.ToDouble(p2.Y - p2.Y), 2));
        }

        /// <summary>
        /// <para>Určí vzdialenosť medzi dvoma bodmi pomocou euklidovskej geomterie.</para>
        /// </summary>
        /// <param name="pointA">Bod X</param>
        /// <param name="pointB">Bod Y</param>
        /// <returns></returns>
        static public double euclidDistance(double[] pointA, double[] pointB)
        {
            double d1 = pointA[0] - pointB[0];
            double d2 = pointA[1] - pointB[1];

            return Math.Sqrt(d1 * d1 + d2 * d2);
        }

    }
}
