﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.Measurements;
using Diplo.Configs;
using Diplo.Libs.Geometry;

namespace Diplo.Measurements
{
    /// <summary>
    /// Trieda porovna, ako velmi su si dve cesty podobne (vracia percenta) <br />
    /// V nasom pripade porovnavame algo / mm cestu s starou cestou
    /// </summary>
    class PathEquality
    {
        private static int evaluationCount = 0;
        private static int onPath = 0;

        public static void evaluate(Graphics g, Stack<Point> newPath, Stack<Point> oldPath, int mode)
        {
            if (newPath.Count > 0 && oldPath.Count > 0)
            {
                PathEquality.init();
                PathEquality.compute(g, newPath, oldPath, LineSegment.pathLength(newPath) / 2, 0);

                PathEquality.update(mode);
            }

        }

        private static void init()
        {
            PathEquality.evaluationCount = 0;
            PathEquality.onPath = 0;
        }

        private static void update(int mode)
        {
            //if (PathEquality.evaluationCount > 0)
            //{
                double a = PathEquality.onPath;
                double b = PathEquality.evaluationCount;
                if (mode == 1)
                {
                    MeasurementBin.PathEqualityMMCountValue++;
                    MeasurementBin.PathEqualityMMLastValue = a / b * 100;

                    MeasurementBin.PathEqualityMMAll.Add(a / b * 100);
                    MeasurementBin.PathEqualityMMTogetherValue += a / b * 100;
                }

                if (mode == 2)
                {
                    MeasurementBin.PathEqualityAlgoCountValue++;
                    MeasurementBin.PathEqualityAlgoLastValue = a / b * 100;
                    MeasurementBin.PathEqualityAlgoAll.Add(a / b * 100);
                    MeasurementBin.PathEqualityAlgoTogetherValue += a / b * 100;
                }
            //}
            
        }

        private static void compute(Graphics g, Stack<Point> newPath, Stack<Point> oldPath, double middleLength, int currentDepth)
        {
            currentDepth++;
            if (currentDepth > Config.PathEqualityMaxDepth)
            {
                return;
            }
            PathEquality.evaluationCount++;

            double tempLength = 0;
            double OldTempLength = 0;
            int afterMiddlePoint = -1;

            for (int i = 1; i < newPath.Count(); i++)
            {

                Point p1 = newPath.ElementAt(i - 1);
                Point p2 = newPath.ElementAt(i);

                double dx = Math.Pow(Math.Abs(p1.X - p2.X), 2);
                double dy = Math.Pow(Math.Abs(p1.Y - p2.Y), 2);
                tempLength += Math.Floor(Math.Sqrt(dx + dy));

                if (tempLength > middleLength)
                {
                    afterMiddlePoint = i;
                    break;
                }

                OldTempLength = tempLength;
            }


            if (afterMiddlePoint != -1)
            {
                // =====================
                // najdenie stredneho vrcholu
                Point middlePoint = PointLine.computePointOnLineSegmentInDistance(g, newPath, afterMiddlePoint, middleLength - OldTempLength);
                if (middlePoint == null)
                {
                    return;
                }

                // =====================
                // lezi stredovi bod na starej ceste ???
                int evaluationValue = PointLine.computePointToPathDistance(g, middlePoint, oldPath);
                if (evaluationValue < Config.PathEqualityTreshold)
                {
                    PathEquality.onPath++;
                }



                /*
                 * !!! zaujimavy vypis...vypise vzdialenost na pozicii bodov
                string drawString = evaluationValue.ToString();
                System.Drawing.Font drawFont = new System.Drawing.Font("Arial", 12);
                System.Drawing.SolidBrush drawBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Black);
                float x = 150.0F;
                float y = 50.0F;
                System.Drawing.StringFormat drawFormat = new System.Drawing.StringFormat();
                g.DrawString(drawString, drawFont, drawBrush, middlePoint.X, middlePoint.Y, drawFormat);
                */


                // ===========================================
                //  rekuriznve volanie - vytvorime dve cesty - od START po MIDDLE a od MIDDLE po FINAL
                Stack<Point> startPath = new Stack<Point>();
                startPath.Push(new Point(middlePoint.X, middlePoint.Y));
                for (int i = afterMiddlePoint - 1; i >= 0; i--)
                {
                    Point p = newPath.ElementAt(i);
                    startPath.Push(new Point(p.X, p.Y));
                }

                Stack<Point> endPath = new Stack<Point>();
                for (int i = newPath.Count() - 1; i >= afterMiddlePoint; i--)
                {
                    Point p = newPath.ElementAt(i);
                    endPath.Push(new Point(p.X, p.Y));
                }
                endPath.Push(new Point(middlePoint.X, middlePoint.Y));

                compute(g, startPath, oldPath, LineSegment.pathLength(startPath) / 2, currentDepth);
                compute(g, endPath, oldPath, LineSegment.pathLength(endPath) / 2, currentDepth);

            }
        }


    }
}
