﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.Libs.Drawing;
using Diplo.GraphObjects;
using Diplo.Configs;
using Diplo.Libs.Geometry.Smart;
using Diplo.Libs.Geometry;
using Diplo.Libs;

namespace Diplo.MentalMap.CoridorLibs
{
    /// <summary>
    /// <para>Staticka trieda helprov pre mentalnu mapu - koridorov</para>
    /// <para>Obsahuje : Pridanie extra bodu k vrcholu</para>
    /// </summary>
    class CoridorMapAdvanceHelpers
    {

        /// <summary>
        /// <para>Pridanie extra bodu k vrcholu - fix na krajsie koridory</para>
        /// </summary>
        /// <param name="g"></param>
        /// <param name="graf"></param>
        /// <param name="edge"></param>
        /// <param name="poligon"></param>
        internal static void addExtraPointTostart(Graphics g, Graph graf, Edge edge, Stack<Point> poligon)
        {
            // pridame specialny bod k prvemu pociatocnemu bodu
            Point p1 = edge.path.ElementAt(0);
            Point p2 = edge.path.ElementAt(1);
            SmartLineSegment Line = new SmartLineSegment(p1, p2);
            // najst extra bod
            Point newP = Line.getPointOutOfLine(g, 1, graf.getVerticeByIndex(edge.node1Index).size + Config.movementArea / 2);
            poligon.Push(newP);
        }

        /// <summary>
        /// <para>Vymazanie hran koridoru, ktore sa pretinaju
        /// - fix pre krajsie koridory</para>
        /// </summary>
        /// <param name="g"></param>
        /// <param name="poligon"></param>
        internal static void removeCrossingLineSegments(Graphics g, Stack<Point> poligon)
        {
            for (int i = 0; i < poligon.Count - 1; i++)
            {
                Point a = new Point(poligon.ElementAt(i).X, poligon.ElementAt(i).Y);
                Point b = new Point(poligon.ElementAt(i + 1).X, poligon.ElementAt(i + 1).Y);

                for (int j = 0; j < poligon.Count - 1; j++)
                {
                    if (Math.Abs(j - i) >= 2)
                    {

                        Point c = new Point(poligon.ElementAt(j).X, poligon.ElementAt(j).Y);
                        Point d = new Point(poligon.ElementAt(j + 1).X, poligon.ElementAt(j + 1).Y);

                        Boolean intersectCheck = LineSegment.DoLineSegmentsIntersect(a.X, a.Y, b.X, b.Y, c.X, c.Y, d.X, d.Y);
                        if (intersectCheck)
                        {
                            //Console.WriteLine("xxxxxxx");
                            DrawPoint.drawHighligthedPoint(g, a, "Green", 8);
                            DrawPoint.drawHighligthedPoint(g, b, "Green", 8);
                            DrawPoint.drawHighligthedPoint(g, c, "Red", 8);
                            DrawPoint.drawHighligthedPoint(g, d, "Red", 8);

                            i = 0;
                            j = 0;

                            StackHelper.removePointFromList(poligon, d);

                        }

                    }
                }
            }
        }

        internal static void drawDiffPoligon(Graphics g, Edge edge , Stack<Point> zaloha, Stack<Point> pathInCorridor, Stack<Point> diferencePolygon)
        {
            // Vytvorime novy DIFF polygon

            foreach (Point p1 in zaloha)
            {
                Boolean addoToDiff = true;

                foreach (Point p2 in pathInCorridor)
                {
                    if (p1.X == p2.X && p1.Y == p2.Y)
                    {
                        addoToDiff = false;
                    }
                }


                if (addoToDiff)
                {
                    g.FillEllipse(new SolidBrush(Color.Green), p1.X - 5, p1.Y - 5, 10, 10);
                    diferencePolygon.Push(p1);
                }
            }

            foreach (Point p1 in pathInCorridor)
            {
                Boolean addoToDiff = true;

                foreach (Point p2 in zaloha)
                {
                    if (p1.X == p2.X && p1.Y == p2.Y)
                    {
                        addoToDiff = false;
                    }
                }


                if (addoToDiff)
                {
                    g.FillEllipse(new SolidBrush(Color.Pink), p1.X - 5, p1.Y - 5, 10, 10);
                    diferencePolygon.Push(p1);
                }
            }

            foreach (Point p in diferencePolygon)
            {
                //g.FillEllipse(new SolidBrush(Color.Yellow), p.X, p.Y, 7, 7);
            }


            if (diferencePolygon.Count > 0)
            {
                g.FillPolygon(new SolidBrush(ColorTransformer.Color2Transparent(Color.Yellow)), diferencePolygon.ToArray());
            }
            edge.diferencePolygon.Clear();
            StackHelper.copyStackToStackReverse(diferencePolygon, edge.diferencePolygon);
        }

    }
}
