﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.Libs.Drawing;
using System.Windows;
using Diplo.DrawAlgorithms;
using Diplo.Measurements;
using Diplo.Debug;
using Diplo.MentalMap.CoridorObjects;
using Diplo.MentalMap.CoridorLibs;
using Diplo.Libs.Geometry.AlphaHull;
using Diplo.GraphObjects;
using Diplo.Configs;
using Diplo.Libs.Geometry;
using Diplo.Libs;

namespace Diplo.MentalMap
{
    class CoridorMap
    {
        // privatne premenne
        Stack<Point> oldPath = new Stack<Point>();
        public Boolean drawTriangulation = false; // vykreslenie Delaune triangulacie
        public Boolean drawAlphacircles = false; // vykreslenie Alfa Kruznic
        public Boolean drawTriangulation1 = false; // vykreslenie Delaune triangulacie
        public Boolean drawAlphacircles1 = false; // vykreslenie Alfa Kruznic

        /// <summary>
        /// Hlavna metoda pri hladani cesty pomocou koridorov
        /// </summary>
        /// <param name="g"></param>
        /// <param name="graf"></param>
        /// <param name="algo"></param>
        /// <param name="edge"></param>
        /// <param name="path"></param>
        /// <returns></returns>
        public Stack<Point> pathInCoridor(Graphics g, Graph graf, DrawInterface algo, Edge edge, Stack<Point> path)
        {
            if (edge.path.Count() == 0)
            {
                StackHelper.copyStackToStack(path, edge.path);
                return path;
            }
            else
            {
                MyDebug.print("Aplikuje sa druhý model mentálnej mapy.");

                // ******************************************
                // initializacia, zaloha dat, merania s algo cestou
                edge.algoPath.Clear();
                StackHelper.copyStackToStackReverse(path, edge.algoPath);
                edge.oldPath.Clear();
                StackHelper.copyStackToStackReverse(edge.path, edge.oldPath);
                Stack<Point> diferencePolygon = new Stack<Point>();

                LengthProportion.evaluate(g, path, 2);
                Binary.evaluate(g, path, edge.path, 2);
                PathEquality.evaluate(g, path, edge.path, 2);


                // ******************************************
                // vypocitame zakladne podkoridory
                Stack<Coridor> edgeBoxes = new Stack<Coridor>();
                for(int i=1;i<edge.path.Count;i++)
                {
                    Stack<Point> edgeBox = CoridorExtenderHelper.getBoxforTwoPoints(g, edge.path.ElementAt(i - 1), edge.path.ElementAt(i), Config.movementArea);

                    if (Config.drawBasicCorridors) { this.drawEdgeBox(g, edgeBox); }

                    Coridor C = new Coridor();
                    C.size = 30;
                    C.points = edgeBox;
                    C.P1 = edge.path.ElementAt(i - 1);
                    C.P2 = edge.path.ElementAt(i);

                    edgeBoxes.Push(C);
                }
                MyDebug.print("(1/3) Vypocitane pokoridory.");


                // ******************************************
                // rozsirime koridory
                if (Config.extendCoridor)
                {
                    CoridorExtender.extendCoridors(g, this, graf, algo, edge, edgeBoxes);

                    if (Config.drawExtendedCorridors)
                    {
                        for (int i = 1; i < edgeBoxes.Count; i++)
                        {
                            this.drawEdgeBox(g, edgeBoxes.ElementAt(i).points);
                        }
                    }
                }
                MyDebug.print("(2/3) Rozsirenie podkoridorov.");


                // ******************************************
                // spojit podkoridory do koridorov pomocou alfa hran
                Stack<Point> poligon = this.createEdgeMovementCoridor(g, edgeBoxes, edge, graf);
                MyDebug.print("(3/3) Vytvoreny koridor.");

                // ******************************************
                // pridat specialny bod
                if (edge.path.Count > 2)
                {
                    CoridorMapAdvanceHelpers.addExtraPointTostart(g, graf, edge, poligon);
                }

                // ******************************************
                // odstranenie cross hran
                if (poligon.Count > 2)
                {
                    CoridorMapAdvanceHelpers.removeCrossingLineSegments(g, poligon);
                }


                // ******************************************
                // poziadame kresliaci algoritmus o cestu v danom poligone
                Stack<Point> zaloha = new Stack<Point>();
                StackHelper.copyStackToStack(path, zaloha);
                if (edge.coridor.Count > 0)
                {
                    Stack<Point> pathInCorridor = algo.drawIntoPolygon(g, graf, path.First(), path.Last(), edge.coridor);
                    if (pathInCorridor.Count > 0)
                    {
                        MyDebug.print("V koridore bola najdena cesta");
                        // vyhodnotit cestu
                        //Binary.evaluate(g, zaloha, pathInCorridor);

                        // vykreslit poligon medz algo a vyslednou cestou
                        if (Config.drawDiffPolygon)
                        {
                            CoridorMapAdvanceHelpers.drawDiffPoligon(g, edge, zaloha, pathInCorridor, diferencePolygon);
                        }

                        zaloha.Clear();
                        StackHelper.copyStackToStack(pathInCorridor, zaloha);
                    }
                    
                }


                // ******************************************
                // merania s vyslednou cestou (ak sa v poligone nenasla nova cesta, je totozna s algo cestou)
                Binary.evaluate(g, zaloha, edge.path, 1);
                LengthProportion.evaluate(g, zaloha, 1);
                PathEquality.evaluate(g, zaloha, edge.path, 1);

                // ulozime staru cestu
                edge.path.Clear();
                StackHelper.copyStackToStack(zaloha, edge.path);

                // ulozit stary koridor
                edge.oldCoridor.Clear();
                StackHelper.copyStackToStack(edge.coridor, edge.oldCoridor);

                // nakopirujeme poligon
                edge.coridor.Clear();
                StackHelper.copyStackToStack(poligon, edge.coridor);

                return zaloha;
            }


        }

        

        /// <summary>
        /// Vytvorenie CORIDORU - hlavna metoda
        /// Z niekolkych bodov vytvori coridori
        /// </summary>
        /// <param name="g">graficky komponent</param>
        /// <param name="edgeBoxes">mnozina bodov, ktore tvoria koridor</param>
        /// <param name="edge"></param>
        /// <param name="graf"></param>
        /// <returns></returns>
        private Stack<Point> createEdgeMovementCoridor(Graphics g, Stack<Coridor> edgeBoxes, Edge edge, Graph graf)
        {
            Stack<Point> poly = new Stack<Point>();
            Stack<CoridorPoint> coridorPolyLeft = new Stack<CoridorPoint>();
            Stack<CoridorPoint> coridorPolyRight = new Stack<CoridorPoint>();

            Point start = new Point();

            Point p1 = new Point();
            Point p2 = new Point();
            for (int i = 0; i < edgeBoxes.Count; i++)
            {
                if (i == 0)
                {
                    p1 = edgeBoxes.ElementAt(i).points.ElementAt(3);
                    p2 = edgeBoxes.ElementAt(i).points.ElementAt(2);

                    start.X = p2.X;
                    start.Y = p2.Y;

                    poly.Push(new Point(p2.X, p2.Y));
                    poly.Push(new Point(p1.X, p1.Y));

                    coridorPolyLeft.Push(new CoridorPoint(p2.X, p2.Y, 1));
                    coridorPolyLeft.Push(new CoridorPoint(p1.X, p1.Y, 1));
                }

                if (i > 0 && i < edgeBoxes.Count)
                {
                    p2 = edgeBoxes.ElementAt(i).points.ElementAt(2);
                    p1 = edgeBoxes.ElementAt(i).points.ElementAt(3);

                    poly.Push(new Point(p2.X, p2.Y));
                    poly.Push(new Point(p1.X, p1.Y));

                    coridorPolyLeft.Push(new CoridorPoint(p2.X, p2.Y, 1));
                    coridorPolyLeft.Push(new CoridorPoint(p1.X, p1.Y, 1));
                }

            }


            /*if (edge.path.Count > 2)
            {
                Point p1Last = edge.path.ElementAt(edge.path.Count - 2);
                Point p2Last = edge.path.ElementAt(edge.path.Count - 1);

                DrawPoint.drawHighligthedPoint(g, p1Last, "Green");
                DrawPoint.drawHighligthedPoint(g, p2Last, "Green");

                SmartLineSegment Line = new SmartLineSegment(p1Last, p2Last);

                Point newP = Line.getPointOutOfLine(g, 2, graf.getVerticeByIndex(edge.node2Index).size + Config.movementArea / 2);
                DrawPoint.drawHighligthedPoint(g, newP, "Green");

                poly.Push(new Point(newP.X, newP.Y));
                coridorPolyLeft.Push(new CoridorPoint(newP.X, newP.Y, 1));
            }*/
            

            //DrawPoint.drawHighligthedPoint(g, p1, "Green");
            //DrawPoint.drawHighligthedPoint(g, p2, "Green");

            //
            /*SmartLineSegment Line = new SmartLineSegment(p1, p2);
            //Line._drawLine(g);
            Point newP = Line.getPointOutOfLine(g, 1, graf.getVerticeByIndex(edge.node1Index).size + Config.movementArea / 2);
            //DrawPoint.drawHighligthedPoint(g, newP, "Green");

            poligon.Push(newP);*/



            for (int i = edgeBoxes.Count-1; i >= 0; i--)
            {
                p2 = edgeBoxes.ElementAt(i).points.ElementAt(0);

                p1 = edgeBoxes.ElementAt(i).points.ElementAt(1);

                poly.Push(new Point(p2.X, p2.Y));
                poly.Push(new Point(p1.X, p1.Y));


                coridorPolyRight.Push(new CoridorPoint(p2.X, p2.Y, 0));
                coridorPolyRight.Push(new CoridorPoint(p1.X, p1.Y, 0));
                
            }

            // ******************************************
            // vytvorenie z vyslednej mnoziny koridor
            if (Config.coridorCreateType == 0)
            {
                // pouzitie ALFA HRAN
                return this.findCorridor(g, poly, edgeBoxes, edge, coridorPolyLeft, coridorPolyRight);
            }
            else
            {
                // pouzitie KONVEXNEHO OBALU
                return ConvexHull.computeConvexHull(g, poly);
            }
            
        }

        /// <summary>
        /// Hlada spravny koridor pomocou alfa hran - spravny koridor je : polynom, ktori obsahuje zaciatocny a koncovy vrchol
        /// Hladanie sa deje pomocou znizovania (iterativne) alfa hodnoty pre alfa kruznice
        /// </summary>
        /// <param name="g"></param>
        /// <param name="poly"></param>
        /// <param name="edgeBoxes"></param>
        /// <param name="edge"></param>
        /// <param name="coridorPolyLeft"></param>
        /// <param name="coridorPolyRight"></param>
        /// <returns></returns>
        private Stack<Point> findCorridor(Graphics g, Stack<Point> poly, Stack<Coridor> edgeBoxes, Edge edge, Stack<CoridorPoint> coridorPolyLeft, Stack<CoridorPoint> coridorPolyRight)
        {

            double polomer = Config.alpha;
            double alpha = -1 / polomer;

            List<Diplo.Libs.Geometry.AlphaHull.Geometry.Edge> edges = new List<Diplo.Libs.Geometry.AlphaHull.Geometry.Edge>();
            Stack<Point> candidateCorridor = new Stack<Point>();

            if (this.drawAlphacircles || this.drawTriangulation)
            {
                if (this.drawAlphacircles)
                {
                    this.drawAlphacircles1 = true;
                }
                if (this.drawTriangulation)
                {
                    this.drawTriangulation1 = true;
                }

                edges = AlphaHullObject.createAlphaHull(this, g, poly, alpha);
                candidateCorridor = AlphaEdgesToPointSequenceConvertor.processAlfaEdgesToPoints(g, poly, edges, edgeBoxes, coridorPolyLeft, coridorPolyRight);

                this.drawAlphacircles1 = false;
                this.drawTriangulation1 = false;
            }
            else
            {
                edges = AlphaHullObject.createAlphaHull(this, g, poly, alpha);
                candidateCorridor = AlphaEdgesToPointSequenceConvertor.processAlfaEdgesToPoints(g, poly, edges, edgeBoxes, coridorPolyLeft, coridorPolyRight);

            }

            Console.WriteLine(edges.Count);
            for (int i = 0; i < edges.Count; i++)
            {

                /*Point p1 = edge.path.ElementAt(edges.ElementAt(i).p1);
                Point p2 = edge.path.ElementAt(edges.ElementAt(i).p2);

                DrawPoint.drawHighligthedPoint(g, p1);*/

                //g.DrawLine(System.Drawing.Pens.Green, p1, p2);
            }


            Point startPoint = edge.path.ElementAt(0);
            Point endPoint = edge.path.ElementAt(edge.path.Count - 1);

            Point endPoint2 = edge.path.ElementAt(edge.path.Count - 1);
            endPoint2.X = endPoint2.X - 20;
            endPoint2.Y = endPoint2.Y - 20;
            Point endPoint3 = edge.path.ElementAt(edge.path.Count - 1);
            endPoint3.X = endPoint3.X + 20;
            endPoint3.Y = endPoint3.Y - 20;
            Point endPoint4 = edge.path.ElementAt(edge.path.Count - 1);
            endPoint4.X = endPoint4.X + 20;
            endPoint4.Y = endPoint4.Y + 20;
            Point endPoint5 = edge.path.ElementAt(edge.path.Count - 1);
            endPoint5.X = endPoint5.X - 20;
            endPoint5.Y = endPoint5.Y + 20;

            Point startPoint2 = edge.path.ElementAt(0);
            startPoint2.X = startPoint2.X - 20;
            startPoint2.Y = startPoint2.Y - 20;
            Point startPoint3 = edge.path.ElementAt(0);
            startPoint3.X = startPoint3.X + 20;
            startPoint3.Y = startPoint3.Y - 20;
            Point startPoint4 = edge.path.ElementAt(0);
            startPoint4.X = startPoint4.X + 20;
            startPoint4.Y = startPoint4.Y + 20;
            Point startPoint5 = edge.path.ElementAt(0);
            startPoint5.X = startPoint5.X - 20;
            startPoint5.Y = startPoint5.Y + 20;

            int count = 0;

            polomer = Config.alpha;
            
            alpha = 1;
            while (count < 1000)
            {

                Boolean check1 = PointInPolygon.isPointInPolygon(g, candidateCorridor, startPoint2) || PointInPolygon.isPointInPolygon(g, candidateCorridor, startPoint3) ||
                    PointInPolygon.isPointInPolygon(g, candidateCorridor, startPoint4) || PointInPolygon.isPointInPolygon(g, candidateCorridor, startPoint5);

                Boolean check2 = PointInPolygon.isPointInPolygon(g, candidateCorridor, endPoint2) || PointInPolygon.isPointInPolygon(g, candidateCorridor, endPoint3) ||
                    PointInPolygon.isPointInPolygon(g, candidateCorridor, endPoint4) || PointInPolygon.isPointInPolygon(g, candidateCorridor, endPoint5);

                

                if (check1 == false || check2 == false)
                {
                    //polomer++;
                    polomer = polomer + 10;

                    alpha = -1 / polomer;

                    edges = AlphaHullObject.createAlphaHull(this, g, poly, alpha);
                    candidateCorridor = AlphaEdgesToPointSequenceConvertor.processAlfaEdgesToPoints(g, poly, edges, edgeBoxes, coridorPolyLeft, coridorPolyRight);
                }
                else
                {// ak ano, koncim uz
                    break;
                }

                count++;
            }

            if (this.drawAlphacircles || this.drawTriangulation)
            {
                if (this.drawAlphacircles)
                {
                    this.drawAlphacircles1 = true;
                }
                if (this.drawTriangulation)
                {
                    this.drawTriangulation1 = true;
                }

                AlphaHullObject.createAlphaHull(this, g, poly, alpha);

                this.drawAlphacircles1 = false;
                this.drawTriangulation1 = false;
            }

            Config.alphaReal = polomer;

            // pridat extra bod na zaciatok / koniec

            /*int d = TwoPoints.manhatan_distance(candidateCorridor.First(), candidateCorridor.Last());

            DrawPoint.drawHighligthedPoint(g, candidateCorridor.First(), "Green");
            DrawPoint.drawHighligthedPoint(g, candidateCorridor.Last(), "Green");

            
            Console.WriteLine("xxxx" + d/2);
            Point p = LineSegment.middlePoint(candidateCorridor.First(), candidateCorridor.Last(), d / 2);
            DrawPoint.drawHighligthedPoint(g, p, "red");*/


            return candidateCorridor;
        }
        

        /// <summary>
        /// 
        /// </summary>
        /// <param name="g"></param>
        /// <param name="edgeBox"></param>
        public void drawEdgeBox(Graphics g, Stack<Point> edgeBox)
        {
            Point[] curve = {edgeBox.ElementAt(0), edgeBox.ElementAt(1), edgeBox.ElementAt(2), edgeBox.ElementAt(3)};
            g.FillPolygon(new SolidBrush(ColorTransformer.Color2Transparent(Color.Blue)), curve);

            foreach (Point p in edgeBox)
            {
                DrawPoint.drawPoint(g, p);
            }
        }

        public void drawEdgeBoxRed(Graphics g, Stack<Point> edgeBox)
        {
            Point[] curve = { edgeBox.ElementAt(0), edgeBox.ElementAt(1), edgeBox.ElementAt(2), edgeBox.ElementAt(3) };
            g.FillPolygon(new SolidBrush(ColorTransformer.Color2Transparent(Color.Blue)), curve);

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="g"></param>
        /// <param name="edgeBox"></param>
        private void drawVerticeBox(Graphics g, Stack<Point> edgeBox)
        {

            Point[] curve = { edgeBox.ElementAt(0), edgeBox.ElementAt(1), edgeBox.ElementAt(2), edgeBox.ElementAt(3) };
            //g.DrawPolygon(System.Drawing.Pens.DarkBlue, curve);
            g.FillPolygon(new SolidBrush(ColorTransformer.Color2Transparent(Color.Green)), curve);

        }

    }
}
