﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.DrawAlgorithms;
using Diplo.Libs.Geometry;
using Diplo.Libs.Drawing;
using Diplo.MentalMap;
using Diplo.GraphObjects;
using Diplo.Configs;

namespace Diplo.MentalMap.CoridorObjects
{
    static class CoridorExtender
    {
        private static Boolean drawColiding = false;

        /// <summary>
        /// Tato metoda vazme body tvoriace zakladny koridor a nafukne ho.
        /// Hladaju, ci sa niekde v koridore nachadza vrchol, az ano bude tato "polovica koridoru nafuknuta"
        /// </summary>
        /// <param name="g"></param>
        /// <param name="CoridorMap"></param>
        /// <param name="graf"></param>
        /// <param name="algo"></param>
        /// <param name="edge"></param>
        /// <param name="edgeBoxes"></param>
        internal static void extendCoridors(Graphics g, CoridorMap CoridorMap, Graph graf, DrawInterface algo, Edge edge, Stack<Coridor> edgeBoxes)
        {
            // inicializacia pola
            int[,] extendCheck = CoridorExtender.initialiseEdgeboxVerticeArray(graf.vertices.Count, edgeBoxes.Count);
         
            // najdeme dvojicu vrchol / edgebox : edgebox budeme rozisrovat, pretoze vrchol do neho zasahuje
            int[] colidingVertexEdgeBoxPair = CoridorExtenderSteps.findOneColidingVertex(g, graf, algo, edge, edgeBoxes, extendCheck);
            while (colidingVertexEdgeBoxPair[0] != -1 && colidingVertexEdgeBoxPair[1] != -1)
            {
                CoridorExtenderSteps.extendEdgeboxByVertice(g, graf, algo, edge, edgeBoxes, colidingVertexEdgeBoxPair);
                
                // najdeme dalsiu dvojicu - vrchol a edgeBox
                colidingVertexEdgeBoxPair = CoridorExtenderSteps.findOneColidingVertex(g, graf, algo, edge, edgeBoxes, extendCheck);
            }
        }

        /// <summary>
        /// Pre dany graf (dany ako pocet vrcholov) a cestu (dana ako pocet EdgeBox-ov) metoda vrati nainicializovane kontrolne pole.
        /// Pole je dvojrozmerne a je pouzivane na overenie, ci dany vrchol uz bol kvoly EdgeBox-u rozsirovany.
        /// (zabranuje sa cyklickosti)
        /// </summary>
        /// <param name="verticesCount"></param>
        /// <param name="edgeboxCount"></param>
        /// <returns></returns>
        private static int[,] initialiseEdgeboxVerticeArray(int verticesCount, int edgeboxCount)
        {
            int[,] extendCheck = new int[verticesCount, edgeboxCount];

            for (int i = 0; i < verticesCount; i++)
            {
                for (int j = 0; j < edgeboxCount; j++)
                {
                    extendCheck[i, j] = 0;
                }
            }

            return extendCheck;
        }


        /// <summary>
        /// Metoda zisti, ktore vrcholy z grafu su kolidujuce - nachadzaju sa v ceste / do nej zasahuju
        /// Vracia pole integerov - indexy na tieto vrcholy
        /// </summary>
        /// <param name="g">Graficky komponent</param>
        /// <param name="algo">vykreslovaci algoritmus</param>
        /// <param name="graf">Graf</param>
        /// <param name="edge">Hrana</param>
        /// <param name="edgeBoxes">EdgeBox</param>
        /// <param name="currentEdgeBoxIndex">CIslo edgeBoxu, ktoru overujeme</param>
        /// <returns></returns>
        internal static List<int> computeWhichVertexAreColiding(Graphics g, DrawInterface algo, Graph graf, Edge edge, Stack<Coridor> edgeBoxes, int currentEdgeBoxIndex)
        {
            List<int> vertexToExpand = new List<int>();

            //for (int i = 0; i < 2; i++)
            for (int i = 0; i < graf.vertices.Count; i++)
            {
                // pre kazdy jeden vrchol zistime, ci nahodou niektory bod z jeho kraju nelezi v niektorom boxe
                // ! vrchol ale nemoze byt ten, ktory tvori cestu
                if (graf.vertices.ElementAt(i).index != edge.node1Index && graf.vertices.ElementAt(i).index != edge.node2Index)
                {
                    Stack<Point> verticeBox = EdgeBoxHelper.getBoxForVertice(graf.vertices.ElementAt(i));

                    int isBoxInSomeEdge = EdgeBoxHelper.checkEdgeBox(verticeBox, edgeBoxes.ElementAt(currentEdgeBoxIndex).points);
                    if (isBoxInSomeEdge != 0 && !edgeBoxes.ElementAt(currentEdgeBoxIndex).isResised)
                    {
                        if (drawColiding) { EdgeBoxHelper.drawVertexColiding(g, verticeBox, edgeBoxes.ElementAt(currentEdgeBoxIndex).points); }
                        vertexToExpand.Add(i);

                    } // ! ak treba dany edgeBox rozsirit

                } // ! pre kazdy vrchol okrem prveho / posledneho hrany

            } // ! pre kazdvy vrchol grafu

            return vertexToExpand;
        }

        /// <summary>
        /// Metoda zisti, ci dany vrchol (dany ako jeho VerticeBox) je kolidujuci s EdgeBox-om.
        /// </summary>
        /// <param name="verticeBox"></param>
        /// <param name="edgeBoxes"></param>
        /// <returns></returns>
        internal static Boolean isVerticeFullInEdgeBox(Stack<Point> verticeBox, Stack<Coridor> edgeBoxes)
        {

            for (int i = 0; i < verticeBox.Count; i++)
            {

                Boolean isInSomeEdgeBox = false;

                for (int j = 0; j < edgeBoxes.Count; j++)
                {


                    if (CoridorExtenderHelper.isPointInEdgeBox(verticeBox.ElementAt(i), edgeBoxes.ElementAt(j).points))
                    {
                        isInSomeEdgeBox = true;
                        break;
                    }
                }

                
                if (isInSomeEdgeBox)
                {
                }
                else
                {
                    return false;
                }
                
            }

            return true;
        }

        

        



        


    }
}
