﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.Libs.Drawing;
using Diplo.GraphObjects;
using Diplo.Libs.Geometry;

namespace Diplo.MentalMap.CoridorObjects
{
    static class CoridorExtenderHelper
    {

        /// <summary>
        /// Posunie vrchol - oznaceny indexom z intervalu (0, 3) o vzdialenost danu ako parameter
        /// Posuvany vrchol - resp. jeho x/y osu - ziska z edgeBoxu. Vrchol je do edgeBoxu aj aktualizovany
        /// Poradie : 
        /// 0 a 3 - "prava strana"
        /// 1 a 2 - "lava strana"
        /// </summary>
        /// <param name="g"></param>
        /// <param name="edgeBox"></param>
        /// <param name="pointToMove"></param>
        /// <param name="distance"></param>
        internal static void moveOnePoint(Graphics g, Coridor edgeBox, int pointToMove, int distance)
        {
            Point A = new Point();
            Point B = new Point();
            if (pointToMove == 0)
            {
                A = new Point(edgeBox.P1.X, edgeBox.P1.Y);
                B = new Point(edgeBox.points.ElementAt(0).X, edgeBox.points.ElementAt(0).Y);
            }
            if (pointToMove == 3)
            {
                A = new Point(edgeBox.P2.X, edgeBox.P2.Y);
                B = new Point(edgeBox.points.ElementAt(1).X, edgeBox.points.ElementAt(1).Y);
            }

            if (pointToMove == 1)
            {
                A = new Point(edgeBox.P1.X, edgeBox.P1.Y);
                B = new Point(edgeBox.points.ElementAt(3).X, edgeBox.points.ElementAt(3).Y);
            }

            if (pointToMove == 2)
            {
                A = new Point(edgeBox.P2.X, edgeBox.P2.Y);
                B = new Point(edgeBox.points.ElementAt(2).X, edgeBox.points.ElementAt(2).Y);
            }

            Point C = LineExtender.extendLineSegment(g, A, B, distance);

            System.Drawing.Point p;
            if (pointToMove == 0)
            {
                p = edgeBox.points.ElementAt(0);
                p.X = C.X;
                p.Y = C.Y;
                addNewPointToEdgeBox(edgeBox, p, 0);
            }
            if (pointToMove == 3)
            {
                p = edgeBox.points.ElementAt(1);
                p.X = C.X;
                p.Y = C.Y;
                addNewPointToEdgeBox(edgeBox, p, 1);
            }


            if (pointToMove == 1)
            {
                p = edgeBox.points.ElementAt(3);
                p.X = C.X;
                p.Y = C.Y;
                addNewPointToEdgeBox(edgeBox, p, 3);
            }

            if (pointToMove == 2)
            {
                p = edgeBox.points.ElementAt(2);
                p.X = C.X;
                p.Y = C.Y;
                addNewPointToEdgeBox(edgeBox, p, 2);
            }

        }


        internal static void addNewPointToEdgeBox(Coridor edgeBox, System.Drawing.Point point, int positionOfNew)
        {

            Stack<Point> points = new Stack<Point>();
            if (positionOfNew == 0) { points.Push(point); }
            else { points.Push(edgeBox.points.ElementAt(0)); }

            if (positionOfNew == 1) { points.Push(point); }
            else { points.Push(edgeBox.points.ElementAt(1)); }

            if (positionOfNew == 2) { points.Push(point); }
            else { points.Push(edgeBox.points.ElementAt(2)); }

            if (positionOfNew == 3) { points.Push(point); }
            else { points.Push(edgeBox.points.ElementAt(3)); }
            
            
            // premaze edgeBox
            edgeBox.points.Clear();
            // pridame nanovo
            edgeBox.points.Push(points.ElementAt(0));
            edgeBox.points.Push(points.ElementAt(1));
            edgeBox.points.Push(points.ElementAt(2));
            edgeBox.points.Push(points.ElementAt(3));
        }

        /// <summary>
        /// Zisti, ci sa dany bod P v edgeBoxe nachadza
        /// </summary>
        /// <param name="P"></param>
        /// <param name="edgeBox"></param>
        /// <returns></returns>
        internal static Boolean isPointInEdgeBox(Point P, Stack<Point> edgeBox)
        {
            Boolean allOnLeft = true;
            // zoberieme jednu z 4 hran edgebox a bod
            for (int l = 0; l < edgeBox.Count - 1; l++)
            {
                if (!PointPosition.isOnTheLeft(P.X, P.Y, edgeBox.ElementAt(l), edgeBox.ElementAt(l + 1)))
                {

                    allOnLeft = false;
                }
            }
            if (!PointPosition.isOnTheLeft(P.X, P.Y, edgeBox.ElementAt(edgeBox.Count - 1), edgeBox.ElementAt(0)))
            {

                allOnLeft = false;
            }
            
            return allOnLeft;
        }

        /// <summary>
        /// Najde najblizsi bod v edgeBoxe (4 body v priestore) k vrcholu z grafu (taktiez bod v pristore)
        /// </summary>
        /// <param name="g"></param>
        /// <param name="graf">graf, ktory obsahuje vsetky vrcholy - v zozname</param>
        /// <param name="edgeBoxes">zoznam edgeBox-ov</param>
        /// <param name="verticeIndex">index vrcholu z grafu</param>
        /// <param name="edgeBoxIndex">index edgeBoxu</param>
        /// <returns></returns>
        internal static int findClosesPointFromEdgeBoxToVertice(Graphics g, Graph graf, Stack<Coridor> edgeBoxes, int verticeIndex, int edgeBoxIndex)
        {
            Point problemVertice = new Point(Convert.ToInt32(Math.Round(graf.vertices.ElementAt(verticeIndex).pos[0])),
                Convert.ToInt32(Math.Round(graf.vertices.ElementAt(verticeIndex).pos[1])));

            double[] P1 = new double[2];
            P1[0] = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(0).X;
            P1[1] = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(0).Y;
            double[] P2 = new double[2];
            P2[0] = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(1).X;
            P2[1] = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(1).Y;
            double[] P3 = new double[2];
            P3[0] = problemVertice.X;
            P3[1] = problemVertice.Y;
            double dist1 = PointLine.LineToPointDistance2D(P1, P2, P3, true);
            
            P1[0] = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(2).X;
            P1[1] = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(2).Y;
            P2[0] = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(3).X;
            P2[1] = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(3).Y;
            double dist2 = PointLine.LineToPointDistance2D(P1, P2, P3, true);

            if (dist1 < dist2)
            {
                //DrawPoint.drawHighligthedPoint(g, edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(0), "Blue");
                //DrawPoint.drawHighligthedPoint(g, edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(1), "Blue");

                Point p = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(0);
                int distance1 = TwoPoints.manhatan_distance(problemVertice, p);
                p = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(1);
                int distance2 = TwoPoints.manhatan_distance(problemVertice, p);
                if (distance1 < distance2)
                {
                    return 0;
                }
                else
                {
                    return 1;
                }
            }
            else
            {
                //DrawPoint.drawHighligthedPoint(g, edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(2), "Blue");
                //DrawPoint.drawHighligthedPoint(g, edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(3), "Blue");

                Point p = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(2);
                int distance1 = TwoPoints.manhatan_distance(problemVertice, p);
                p = edgeBoxes.ElementAt(edgeBoxIndex).points.ElementAt(3);
                int distance2 = TwoPoints.manhatan_distance(problemVertice, p);
                if (distance1 < distance2)
                {
                    return 2;
                }
                else
                {
                    return 3;
                }
            }
        }


        /// <summary>
        /// Pre dva body z cesty vytvorime edge box
        /// </summary>
        /// <param name="g"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="size"></param>
        /// <returns></returns>
        internal static Stack<Point> getBoxforTwoPoints(Graphics g, Point x, Point y, int size)
        {
            int thickness = size;
            Point R1 = new Point();
            Point R2 = new Point();
            Point R3 = new Point();
            Point R4 = new Point();

            int width = y.X - x.X;
            int height = y.Y - x.Y;

            double dx = Math.Pow(Math.Abs(y.X - x.X), 2);
            double dy = Math.Pow(Math.Abs(y.Y - x.Y), 2);
            int tempLength = Convert.ToInt32(Math.Floor(Math.Sqrt(dx + dy)));

            int xS = (thickness * height / tempLength) / 2;
            int yS = (thickness * width / tempLength) / 2;

            R1.X = x.X - xS;
            R1.Y = x.Y + yS;
            R2.X = x.X + xS;
            R2.Y = x.Y - yS;
            R3.X = y.X + xS;
            R3.Y = y.Y - yS;
            R4.X = y.X - xS;
            R4.Y = y.Y + yS;

            Stack<Point> ret = new Stack<Point>();

            ret.Push(R2);
            ret.Push(R3);
            ret.Push(R4);
            ret.Push(R1);

            return ret;
        }

    }
}
