﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.DrawAlgorithms;
using Diplo.MentalMap.CoridorObjects.CoridorExtenders;
using Diplo.GraphObjects;
using Diplo.Configs;
using Diplo.Libs.Geometry;

namespace Diplo.MentalMap.CoridorObjects
{
    internal class CoridorExtenderSteps
    {
        

        /// <summary>
        /// <para>**** KROK 1 *** </para>
        /// <para>Hladame vrchol (z grafu) a edgebox, ktory budeme rozisrovat. Pre vrchol musi platit :</para>
        /// <para> - zasahuje aspon do jedneho edgeboxu - je kolidujuci s nim (tento edgebox je vrateny ak odvojica s vrcholom)</para>
        /// <para> - aspon jeden z jeho rohovych bodov nelezi v niektorom edgeboxu (lubovolnom) - vrchol nie je pohlteny edgeboxami</para>
        /// <para> - dany koridor sa kvoly tomu vrcholu nerozsiroval (zabranenie zacykleniu)</para>
        /// </summary>
        /// <param name="g">graficky komponent</param>
        /// <param name="graf">graf instancia (obsahuje vrcholy)</param>
        /// <param name="algo">kresliaci algoritmus</param>
        /// <param name="edge">hrana</param>
        /// <param name="edgeBoxes">edgeboxy pre hranu</param>
        /// <param name="extendCheck"></param>
        /// <returns>dvojica int[] v poradi : ret[0] je index vrcholu, ret[1] je index edgeboxu</returns>
        internal static int[] findOneColidingVertex(Graphics g, Graph graf, DrawInterface algo, Edge edge, Stack<Coridor> edgeBoxes, int[,] extendCheck)
        {
            int[] colidingVertexEdgeBoxPair = new int[2];
            colidingVertexEdgeBoxPair[0] = -1;
            colidingVertexEdgeBoxPair[1] = -1;

            // pre kazdy edgebox cesty budeme hladat kolidujuce vrcholy
            for (int i = 0; i < edgeBoxes.Count; i++)
            {
                // pre dany edgebox si vyberiem vsetky koliduju vrcholy z grafu
                List<int> colidingVertexes = CoridorExtender.computeWhichVertexAreColiding(g, algo, graf, edge, edgeBoxes, i);

                // pre kazdy kolidujuci vrchol overim, ci nahodou cely nelezi v koridoroch (inych edgeboxoch)
                foreach (int vertex in colidingVertexes)
                {
                    Stack<Point> verticeBox = EdgeBoxHelper.getBoxForVertice(graf.vertices.ElementAt(vertex)); // box pre vrchol (4 vrcholy, ktore ho ohranicuju)
                    Boolean isIn = CoridorExtender.isVerticeFullInEdgeBox(verticeBox, edgeBoxes); // kontrola, ci vrchol koliduje s EdgeBox-om
                    
                    // ak vrchol nie je uz pohlteny edgeboxami a nebol kvoly danemu edgeboxu este rozisrovany
                    if (!isIn && extendCheck[vertex, i] == 0)
                    {
                        extendCheck[vertex, i] = 1; // oznacit

                        colidingVertexEdgeBoxPair[0] = vertex;
                        colidingVertexEdgeBoxPair[1] = i;

                        return colidingVertexEdgeBoxPair;
                    }

                }
            }

            return colidingVertexEdgeBoxPair;
        }


        /// <summary>
        /// <para>**** KROK 2 *** </para>
        /// <para>Rozsirenie edgeboxu a vrchol</para>
        /// </summary>
        /// <param name="g"></param>
        /// <param name="graf"></param>
        /// <param name="algo"></param>
        /// <param name="edge"></param>
        /// <param name="edgeBoxes"></param>
        /// <param name="colidingVertexEdgeBoxPair"></param>
        internal static void extendEdgeboxByVertice(Graphics g, Graph graf, DrawInterface algo, Edge edge, Stack<Coridor> edgeBoxes, int[] colidingVertexEdgeBoxPair)
        {
            int nearestOneIndex = CoridorExtenderHelper.findClosesPointFromEdgeBoxToVertice(g, graf, edgeBoxes, colidingVertexEdgeBoxPair[0], colidingVertexEdgeBoxPair[1]);
            int newSize = graf.vertices.ElementAt(colidingVertexEdgeBoxPair[0]).size * 2 + Config.vertexArea * 2 + 10;

            Stack<Point> extendedEdgeBox = CoridorExtenderSideMover.extend(g, edgeBoxes.ElementAt(colidingVertexEdgeBoxPair[1]), newSize, nearestOneIndex);
        }

    }
}
