﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using Diplo.Measurements;
using Diplo.Libs.Drawing;
using Diplo.DrawAlgorithms;
using System.Drawing.Drawing2D;
using Diplo.Debug;
using Diplo.GraphObjects;
using Diplo.Configs;
using Diplo.Libs;

namespace Diplo.MentalMap
{
    /// <summary>
    /// Hlavna kresliaca trieda. Poskytuje metody pre rychle kreslenie Grafu a predpocitanych ciest / pocitanie novych ciest s vhodnou mentalnou mapou.
    /// </summary>
    class MainDraw
    {
        /* Kresliace algoritmi */
        private AStar Astar = new AStar();
        private Lee Lee = new Lee();

        /* Mentalne mapy */
        public MaxDifMap LeeMMMaxDif;
        public LeeMMCM LeeMMCM = new LeeMMCM();
        public Binary binaryHeuristic = new Binary();
        public CoridorMap coridor = new CoridorMap();

        /* Privatne premenne */
        private Graphics g;
        private bool repaint = false;

        /* Verejne premenne */
        public Graph graf = new Graph();
        public Stack<Point> path = new Stack<Point>();
        public int mentalMapType = 0;
        public int algoType = 0;

        /* konfiguracie */
        public bool drawVertexAreas = true; // vykreslit plochu okolo vrcholov, kde by nemali zasahovat hrany ?
        public bool drawCirle = false; // vrcholom sa ma vykreslit kruzok ?
        public bool drawFillCirle = true; // vrcholy maju byt plne ?
        public bool edgePoints = true; // body, kde sa hrana lame - zaroven aj body pre mentaln muapu hran


        public bool drawOriginal = false;
        public bool drawOld = false;
        public bool drawDiffPolygon = false;


        public int vertexArea = 10;
        public int vertexSize = 10;

        // =======================


        public MainDraw()
        {
            this.LeeMMMaxDif = new MaxDifMap(this);
        }

        /// <summary>
        /// Reset grafu do zaciatocneho stavu
        /// </summary>
        public void resetGrapf()
        {
            MyDebug.print("Reset grafu");
            graf = XmlLoader.loadFromFile();
            repaint = true;
        }

        /// <summary>
        /// <para>Tahanie vrcholu mysou.</para>
        /// </summary>
        /// <param name="X"></param>
        /// <param name="Y"></param>
        /// <returns></returns>
        public Vertex pointDraged(int X, int Y)
        {
            foreach (Vertex v in graf.vertices)
            {
                int x1 = (int)Math.Round(v.pos[0]);
                int y1 = (int)Math.Round(v.pos[1]);
                
                if (X >= (x1) && X <= (x1 + v.size) && Y >= (y1) && Y <= (y1 + v.size))
                {
                    return v;
                }

            }

            return null;
        }

        
        /// <summary>
        /// <para>Funkcia pre vykreslenie grafu.</para>
        /// </summary>
        /// <param name="e">PaintEventArgs e argument ...</param>
        /// <param name="pictureBox1">PictureBox, kam budeme malovat.</param>
        /// <param name="findNewPaths"></param>
        public void draw(System.Windows.Forms.PaintEventArgs e, PictureBox pictureBox1, Boolean findNewPaths = true)
        {
            g = e.Graphics;
            g.SmoothingMode = SmoothingMode.AntiAlias;
            DrawBackground.hex(g, pictureBox1);

            if (!repaint)
            { // prvotne vykreslenie aj s nacitanim z XML poloh
                graf = XmlLoader.loadFromFile();
                repaint = true;
                this.paint();
            }
            else // prekreslenie
            {
                this.paint(findNewPaths);
            }
            
        }


        /// <summary>
        /// Vykreslenie grafu, pricom v tejto metode sa moyu cesty aj prepocitavat na novo.
        /// </summary>
        /// <param name="findNewPaths"></param>
        private void paint(Boolean findNewPaths = true)
        {
            if (findNewPaths)
            {
                MyDebug.print("***");
                MyDebug.print("Štart hľadania nových ciest");
                foreach (Edge edge in graf.edges) // kazdu hranu v grafe vykreslime
                {
                    // spajanie hrany
                    Vertex start = (from vertex in graf.vertices where vertex.index == edge.node1Index select vertex).First();
                    Vertex end = (from vertex in graf.vertices where vertex.index == edge.node2Index select vertex).First();

                    int x = (int)Math.Round(start.pos[0]) + start.size / 2;
                    int y = (int)Math.Round(start.pos[1]) + start.size / 2;
                    int finishX = (int)Math.Round(end.pos[0]) + end.size / 2;
                    int finishY = (int)Math.Round(end.pos[1]) + end.size / 2;

                    if (x == finishX && y == finishY) { }
                    else
                    {
                        // hladanie novej cesty
                        MyDebug.print("Hladanie cesty medzi vrcholmi : " + Convert.ToString(edge.node1Index) + " | " + Convert.ToString(edge.node2Index) );
                        if (this.algoType == 0) { this.path = Astar.draw(g, graf, new Point(x, y), new Point(finishX, finishY), 600); }
                        if (this.algoType == 1) { this.path = Lee.draw(g, graf, new Point(x, y), new Point(finishX, finishY), 600); }

                        if (this.mentalMapType == 0) { }
                        if (this.mentalMapType == 1) { this.path = this.LeeMMMaxDif.maxDiferentPoints(g, graf, Astar, edge, path); }
                        if (this.mentalMapType == 2) { this.path = coridor.pathInCoridor(g, graf, Astar, edge, path); }

                        // nakopirovanei vyslednej cesty
                        if (this.path.Count > 0)
                        {
                            edge.path.Clear();
                            StackHelper.copyStackToStack(this.path, edge.path);
                        }

                    }
                }

                MyDebug.print("Koniec hľadania nových ciest");
            }

            // vykreslit graf
            this.drawGraphInCache(g);
        }




        /// <summary>
        /// <para>Iba vykreslenie existujuceho grafu</para>
        /// <para>Metóda vykreslí VRCHOLY a HRANY grafu, pricom hrany nepocita - vykresli iba uz ulozeny a vypocitany graf. 
        /// Metoda je vhodna pre casto a rychlo za sebou sa opakujuce udalosti.</para>
        /// </summary>
        /// <param name="g"></param>
        public void drawGraphInCache(Graphics g){

            this.drawGraphVertices(g, this.graf);

            foreach (Edge edge in graf.edges) // kazdu hranu v grafe vykreslime
            {
                if (Config.drawAlgoPath) { DrawPath.pink(g, edge.algoPath); }
                if (Config.drawOldPath) { DrawPath.simple(g, edge.oldPath); }
                if (Config.drawDiffPolygon && edge.diferencePolygon.Count > 0)
                {
                    g.FillPolygon(new SolidBrush(ColorTransformer.Color2Transparent(Color.Blue)), edge.diferencePolygon.ToArray());
                }
                if (Config.showPoligon && edge.oldCoridor.Count > 0)
                {
                    g.FillPolygon(new SolidBrush(ColorTransformer.Color2Transparent(Color.Blue)), edge.oldCoridor.ToArray());
                    DrawPath.blue(g, edge.oldCoridor);
                }

                // spajanie hrany
                Vertex start = (from vertex in graf.vertices where vertex.index == edge.node1Index select vertex).First();
                Vertex end = (from vertex in graf.vertices where vertex.index == edge.node2Index select vertex).First();

                int x = (int)Math.Round(start.pos[0]) + start.size / 2;
                int y = (int)Math.Round(start.pos[1]) + start.size / 2;
                int finishX = (int)Math.Round(end.pos[0]) + end.size / 2;
                int finishY = (int)Math.Round(end.pos[1]) + end.size / 2;

                if (x == finishX && y == finishY) { }
                else
                {

                    this.path = edge.path;
                    Point prevPoint = new Point(0, 0);
                    int a = 0;
                    foreach (Point p in this.path)
                    {
                        if (this.edgePoints)
                        {
                            if (a == 0 || a - 1 == this.path.Count)
                            {
                                g.FillEllipse(new SolidBrush(Color.Red), p.X - 3, p.Y - 3, 6, 6);
                            }
                            else
                            {
                                g.FillEllipse(new SolidBrush(Color.Red), p.X - 3, p.Y - 3, 6, 6);
                            }
                        }

                        if (a == 0 || a - 1 == this.path.Count)
                        {
                            prevPoint = new Point(p.X, p.Y);
                        }
                        else
                        {
                            g.DrawLine(System.Drawing.Pens.Black, prevPoint.X, prevPoint.Y, p.X, p.Y);
                            prevPoint = new Point(p.X, p.Y);
                        }

                        a++;
                    }

                }
            }

        }

        


        /// <summary>
        /// <para>Obycajne prekreslenie grafu - kresli iba HRANY grafu</para>
        /// <para>Tato metoda iba prekresli graf. Predpoklada sa, ze v grafe uz kazda cesta ma najdenu cestu. Metodu je vhodne pouzit pri udalostiach ako
        /// pohyb mysou a pod. - udalosti, ktore sa mozu diat velmi casto a je preto treba rychlo prekreslovat plochu.
        /// </para>
        /// </summary>
        /// <param name="g"></param>
        /// <param name="graph"></param>
        private void drawGraphVertices(Graphics g, Graph graph)
        {
            if (this.drawCirle || this.drawFillCirle)
            {
                foreach (Vertex v in graph.vertices)
                {
                    int x = (int)Math.Round(v.pos[0]);
                    int y = (int)Math.Round(v.pos[1]);

                    SolidBrush blackBrush = new SolidBrush(Color.Black);

                    if (this.drawVertexAreas)
                    {
                        int left = x - vertexArea;
                        int up = y - vertexArea;
                        int right = x + v.size + vertexArea;
                        int down = y + v.size + vertexArea;
                        if (Config.drawVertexBoundary)
                        {
                            g.DrawLine(System.Drawing.Pens.Red, left, up, right, up);
                            g.DrawLine(System.Drawing.Pens.Red, right, up, right, down);
                            g.DrawLine(System.Drawing.Pens.Red, right, down, left, down);
                            g.DrawLine(System.Drawing.Pens.Red, left, down, left, up);
                        }
                    }

                    // elipsa - vrchol
                    if (this.drawCirle) { g.DrawEllipse(System.Drawing.Pens.Black, x, y, v.size, v.size); }
                    if (this.drawFillCirle) { g.FillEllipse(blackBrush, x, y, v.size, v.size); }
                }
            }

        }

    }
}
