﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using Diplo.Libs.Drawing;
using Diplo.DrawAlgorithms;
using Diplo.Measurements;
using Diplo.Debug;
using Diplo.GraphObjects;
using Diplo.Configs;
using Diplo.Libs;

namespace Diplo.MentalMap
{

    // ====================================================
    // Mentalna Mapa - Model max. poctu rozdielnych bodov
    // == pre Leeho Algoritmus
    // ====================================================

    class MaxDifMap
    {
        // privatne premenne
        private MainDraw MD;
        Stack<Point> oldPath = new Stack<Point>();

        // parametre a nastavenia
        
        
        public MaxDifMap(MainDraw MD)
        {
            this.MD = MD;
        }

        public Stack<Point> maxDiferentPoints(Graphics g, Graph graf, DrawInterface algo, Edge edge, Stack<Point> path)
        {
            // ak este n emam v pamati ziadnu cestu
            if (edge.path.Count() == 0)
            {
                StackHelper.copyStackToStack(path, edge.path);
                return path;
            }
            else
            {
                MyDebug.print("Aplikuje sa prvý model mentálnej mapy.");

                Stack<Point> diferencePolygon = new Stack<Point>();
                LengthProportion.evaluate(g, path, 2);
                Binary.evaluate(g, path, edge.path, 2);
                PathEquality.evaluate(g, path, edge.path, 2);

                // ==================================================
                // inicializacia - > ulozenie starej cesty, algo cesty
                // ==================================================

                edge.algoPath.Clear();
                StackHelper.copyStackToStackReverse(path, edge.algoPath);
                edge.oldPath.Clear();
                StackHelper.copyStackToStackReverse(edge.path, edge.oldPath);

                Stack<Point> zaloha = new Stack<Point>();
                StackHelper.copyStackToStack(path, zaloha);
                

                // inicializacie
                Stack<Point> newPath = new Stack<Point>();
                this.oldPath = edge.path;
                int size = this.oldPath.Count;
                int commonCounter = 0;
                bool common = true;
                Point firstNew = new Point();

                // ==================================================
                // === AK SA ZMENIL PRVY VRCHOL === //
                // ==================================================
                if (edge.path.ElementAt(0).X != path.ElementAt(0).X || edge.path.ElementAt(0).X != path.ElementAt(0).X)
                {
                    this.firstVerticeChange(g, graf, algo, edge, path, newPath);
                    return path;
                }


                // ==================================================
                // === AK SA ZMENIL POSLEDNY VRCHOL === //
                // ==================================================
                if (edge.path.ElementAt(edge.path.Count()-1).X != path.ElementAt(path.Count()-1).X || edge.path.ElementAt(edge.path.Count()-1).X != path.ElementAt(path.Count()-1).X)
                {
                    this.lastVerticeChange(g, graf, algo, edge, path, newPath);
                    return path;
                }


                // ==================================================
                // zmena nastala niekde v strede cesty
                // ==================================================

                // === kopirovanie spolocnych starych a novych bodov
                int changesStarted = -1;
                while (commonCounter < size && common)
                {
                    if(edge.path.Count() > commonCounter && path.Count() > commonCounter )
                    {
                        Point oldP = edge.path.ElementAt(commonCounter);
                        Point newP = path.ElementAt(commonCounter);

                        if (oldP.X != newP.X || oldP.Y != newP.Y) { 
                            common = false;
                            diferencePolygon.Push(oldP);
                            changesStarted = commonCounter;
                        }
                        else
                        {
                            newPath.Push(oldP);
                            firstNew.X = oldP.X; // bod, v ktorom sme prestali sledovat staru cestu
                            firstNew.Y = oldP.Y;
                            commonCounter++;
                        }
                    }
                    else
                    {
                        common = false;
                        changesStarted = commonCounter;
                    }
                }

                // mame povolenych maxDifferent vrcholov, najdeme si teda dalsi vrchol z povodnej cesty
                commonCounter += Config.maxDifferent;
                if (this.oldPath.Count() <= commonCounter) { commonCounter = this.oldPath.Count() - 1; }

                // === Ak som uz na konci starej cesty - v koncovom bode
                if (firstNew.X == this.oldPath.ElementAt(commonCounter).X && firstNew.Y == this.oldPath.ElementAt(commonCounter).Y)
                {
                }
                else // =========== ak nastala nejaka zmena
                {

                    // === pridam vrcholy z novej cesty
                    for (int i = changesStarted; i < commonCounter; i++)
                    {
                        if (i < path.Count())
                        {
                            Point newP = path.ElementAt(i);
                            newPath.Push(newP);

                            diferencePolygon.Push(newP);
                        }

                        
                    }
                    
                    
                    // ak som na konci - koncim
                    if (edge.path.ElementAt(edge.path.Count() - 1).X == newPath.ElementAt(0).X && edge.path.ElementAt(edge.path.Count() - 1).Y == newPath.ElementAt(0).Y)
                    {
                        //System.Console.WriteLine("som na konci !");
                    }
                    else
                    {
                        // najdem najblizsii bod spomedzi bodov starej cesty k poslednemu bodu, kde sme koncili
                        commonCounter = commonCounter - 1;
                        double[] PointC = new double[2];
                        PointC[0] = path.ElementAt(commonCounter).X;
                        PointC[1] = path.ElementAt(commonCounter).Y;

                        double min = -1;
                        int closestOne = -1;
                        for (int i = commonCounter; i < edge.path.Count(); i++)
                        {
                            double[] PointA = new double[2];
                            double[] PointB = new double[2];
                            

                            PointA[0] = edge.path.ElementAt(i - 1).X;
                            PointA[1] = edge.path.ElementAt(i - 1).Y;

                            PointB[0] = edge.path.ElementAt(i).X;
                            PointB[1] = edge.path.ElementAt(i).Y;

                            double res = Libs.Geometry.PointLine.LineToPointDistance2D(PointA, PointB, PointC, true);
                            if (min == -1 || res < min)
                            {
                                min = Math.Floor(res);
                                closestOne = i;
                            }

                        }


                        for (int i = closestOne; i >= commonCounter; i--)
                        {
                            diferencePolygon.Push(edge.path.ElementAt(i));
                        }

                        // najdem cestu medzi poslednym pridanym z novej cesty a najblizsim najdenym
                        Stack<Point> reroutePath = new Stack<Point>();
                        if (path.ElementAt(commonCounter).X == edge.path.ElementAt(closestOne).X && path.ElementAt(commonCounter).Y == edge.path.ElementAt(closestOne).Y) {
                            newPath.Push(path.ElementAt(commonCounter));
                        }
                        else
                        {
                            MyDebug.print("Hľadá sa nopojenie na pokračovanie cesty.");
                            reroutePath = algo.draw(g, graf, new Point(path.ElementAt(commonCounter).X, path.ElementAt(commonCounter).Y), new Point(edge.path.ElementAt(closestOne).X, edge.path.ElementAt(closestOne).Y), 600);
                            foreach (Point p in reroutePath)
                            {
                                newPath.Push(p);
                            }
                        }               
                        

                        for (int i = closestOne; i < this.oldPath.Count(); i++)
                        {
                            Point newP = oldPath.ElementAt(i);
                            newPath.Push(newP);
                        }

                        
                        edge.diferencePolygon.Clear();
                        StackHelper.copyStackToStackReverse(diferencePolygon, edge.diferencePolygon);
                    }
                }

                // vyhodnotenie VYSLEDNEJ cesty (s Mentalnou Mapou)
                Binary.evaluate(g, newPath, edge.path, 1);
                LengthProportion.evaluate(g, newPath, 1);
                PathEquality.evaluate(g, newPath, edge.path, 1);

                // nova cesta pojde do cesty pre vykreslenie
                path.Clear();
                StackHelper.copyStackToStackReverse(newPath, path);
                edge.path.Clear();
                StackHelper.copyStackToStackReverse(newPath, edge.path);

                

                return path;
            }

        }

        /// <summary>
        /// Ak sa zmenil PRVY vrchol hrany, tato metoda vytvori novu hranu
        /// </summary>
        /// <param name="g"></param>
        /// <param name="algo">Vykreslovaci algoritmus</param>
        /// <param name="graf"></param>
        /// <param name="edge">Hrana, ktoru menime - v hrane je ulozena predchadzajuci tvar hrany (pred zmenou)</param>
        /// <param name="path">Cesta, ktoru navrhol vykreslovaci algoritmus pre vykreslenie</param>
        /// <param name="newPath">Nova cesta, ktoru vytvorime</param>
        private void firstVerticeChange(Graphics g, Graph graf, DrawInterface algo, Edge edge, Stack<Point> path, Stack<Point> newPath)
        {
            Vertex start = (from vertex in graf.vertices where vertex.index == edge.node1Index select vertex).First();
            Stack<Point> reroutePath = algo.draw(g, graf, new Point(Convert.ToInt32(start.pos[0]) + start.size / 2, Convert.ToInt32(start.pos[1]) + start.size / 2), 
                                                          new Point(edge.path.ElementAt(1).X, edge.path.ElementAt(1).Y), 600);

            // pridame novy pociatocny vrchol
            newPath.Push(new Point(Convert.ToInt32(start.pos[0]) + start.size / 2, Convert.ToInt32(start.pos[1]) + start.size / 2));
            // kazdy vrchol z reroute cesty pridame
            foreach (Point p in reroutePath)
            {
                newPath.Push(p);
            }

            edge.path.Pop(); // odstranime povodny vrchol, kde zacinala stara cesta
            StackHelper.copyStackToStackReverse(edge.path, newPath);

            // nova cesta pojde do cesty pre vykreslenie
            path.Clear();
            StackHelper.copyStackToStackReverse(newPath, path);
            edge.path.Clear();
            StackHelper.copyStackToStackReverse(newPath, edge.path);
        }

        /// <summary>
        /// Ak sa zmenil POSLEDNY vrchol hrany, tato metoda vytvori novu hranu
        /// </summary>
        /// <param name="g"></param>
        /// <param name="algo">Vykreslovaci algoritmus</param>
        /// <param name="graf">Graf</param>
        /// <param name="edge">Hrana, ktoru menime - v hrane je ulozena predchadzajuci tvar hrany (pred zmenou)</param>
        /// <param name="path">Cesta, ktoru navrhol vykreslovaci algoritmus pre vykreslenie</param>
        /// <param name="newPath">Nova cesta, ktoru vytvorime</param>
        private void lastVerticeChange(Graphics g, Graph graf, DrawInterface algo, Edge edge, Stack<Point> path, Stack<Point> newPath)
        {
            Vertex end = (from vertex in graf.vertices where vertex.index == edge.node2Index select vertex).First();

            StackHelper.copyStackToStackReverse(edge.path, newPath);
            newPath.Pop();
            Stack<Point> reroutePath = algo.draw(g, graf, new Point(edge.path.ElementAt(edge.path.Count() - 2).X, edge.path.ElementAt(edge.path.Count() - 2).Y), 
                                                          new Point(Convert.ToInt32(end.pos[0]), Convert.ToInt32(end.pos[1])), 600);
            foreach (Point p in reroutePath)
            {
                newPath.Push(p);
            }
            newPath.Push(new Point(Convert.ToInt32(end.pos[0]) + end.size / 2, Convert.ToInt32(end.pos[1]) + end.size / 2));


            // nova cesta pojde do cesty pre vykreslenie
            path.Clear();
            StackHelper.copyStackToStackReverse(newPath, path);
            edge.path.Clear();
            StackHelper.copyStackToStackReverse(newPath, edge.path);
        }


    }

}
