﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using Diplo.GraphObjects;

namespace Diplo.MentalMap
{

    // ====================================================
    // Mentalna Mapa - Model obmedzenia pohybu vrcholov
    // == pre Leeho Algoritmus
    // ====================================================

    class LeeMMCM
    {

        int movementArea = 50;
        int granularityOfNeigb = 2;

        Stack<Point> oldPath = new Stack<Point>();

        public Stack<Point> maxConstraints(Graphics g, MainDraw algo, Edge edge, Stack<Point> path)
        {
            if (edge.path.Count() == 0)
            {
                // prekopirujeme cestu do hrany (aby sme si ju pamatali)
                copyStackToStack(path, edge.path);
                return path;
            }
            else
            {
                this.drawPath(g, path);

                System.Console.WriteLine("pocet vrcholov starej : " + edge.path.Count);
                int i = 0;

                Stack<Point> newPath = new Stack<Point>();

                foreach (Point p in path)
                {

                    if (i > 0 && i < edge.path.Count - 2)
                    { // pre kazdy bod (okrem prveho a poslednych dvoch) vykreslime oblast, v ktorej sa mozu hybat
                        this.drawMovemenetAreaForPoint(g, p);
                    }

                    // posun vrcholu
                    //int[] moved = this.movePoint(algo, p, edge.path.ElementAt(edge.path.Count - 1));
                    int[] moved = this.movePointB(algo, path, edge, p);
                    newPath.Push(new Point(moved[0], moved[1]));



                    i++;
                }

                //algo.path.Clear();

                edge.path.Clear();
                copyStackToStack(newPath, edge.path);

                return newPath;
            }
            

        }

        private int[] movePointB(MainDraw algo, Stack<Point> path, Edge edge, Point p)
        {

            double min = -1;
            int closestOne = -1;
            for (int i = 1; i < edge.path.Count(); i++)
            {
                double[] PointA = new double[2];
                double[] PointB = new double[2];
                double[] PointC = new double[2];

                PointA[0] = edge.path.ElementAt(i - 1).X;
                PointA[1] = edge.path.ElementAt(i - 1).Y;

                PointB[0] = edge.path.ElementAt(i).X;
                PointB[1] = edge.path.ElementAt(i).Y;

                PointC[0] = p.X;
                PointC[1] = p.Y;

                double res = Libs.Geometry.PointLine.LineToPointDistance2D(PointA, PointB, PointC, true);
                if (min == -1 || res < min)
                {
                    min = Math.Floor(res);
                    closestOne = i;
                }

            }

            //System.Console.WriteLine("najblizsi je : " + closestOne);



            return this.movePoint(algo, p, edge.path.ElementAt(closestOne));
        }

        private int[] movePoint(MainDraw algo, Point point, Point dest)
        {
            int[] ret = new int[2];
            ret[0] = point.X;
            ret[1] = point.Y;

            int zoneX = point.X;
            int zoneY = point.Y;

            int count = 0;
            Boolean noChange = false;
            while (count < movementArea && !noChange)
            {
                int[,] susedia = this.getNeigh(point);

                // ohodnotime susedov a seba
                for (int i = 0; i < 4; i++)
                {
                    //System.Console.WriteLine(this.manhatanDist(susedia[i,0], susedia[i,1], dest));
                    susedia[i, 2] = this.manhatanDist(susedia[i, 0], susedia[i, 1], dest);
                }
                int myScore = this.manhatanDist(point.X, point.Y, dest);
                //System.Console.WriteLine(myScore);

                // zrusit vrcholy mimo zonu
                for (int i = 0; i < 4; i++)
                {
                    if (susedia[i, 0] < zoneX - movementArea || susedia[i, 0] > zoneX + movementArea) { susedia[i, 2] = -1; }
                    if (susedia[i, 1] < zoneY - movementArea || susedia[i, 1] > zoneY + movementArea) { susedia[i, 2] = -1; }
                    
                }
                // zrusime vrcholy, ktore su v oblasti s niektorym uzlom
                for (int i = 0; i < 4; i++)
                {
                    foreach (Vertex v in algo.graf.vertices)
                    {
                        int x1 = (int)Math.Round(v.pos[0]);
                        int y1 = (int)Math.Round(v.pos[1]);

                        if (susedia[i, 0] == x1 && susedia[i, 1] == y1) { }
                        else
                        {

                            int left = x1 - algo.vertexArea;
                            int up = y1 - algo.vertexArea;
                            int right = x1 + v.size + algo.vertexArea;
                            int down = y1 + v.size + algo.vertexArea;

                            if ((left < susedia[i, 0] && right > susedia[i, 0]) && (up < susedia[i, 1] && down > susedia[i, 1]))
                            {
                                susedia[i, 2] = -1;
                            }

                        }

                    }       
                }
                                                                                                  

                // najdeme, najlepsi posun
                int posun = -1;
                int best = myScore;
                for (int i = 0; i < 4; i++)
                {
                    if (best > susedia[i, 2] && susedia[i, 2] != -1)
                    {
                        posun = i;
                        best = susedia[i, 2];
                    }
                }

                // ak som nasiel lepsiu polohu posuniem sa nan
                if (posun != -1)
                {
                    ret[0] = susedia[posun, 0];
                    ret[1] = susedia[posun, 1];
                    point.X = susedia[posun, 0];
                    point.Y = susedia[posun, 1];
                }
                else
                {
                    //System.Console.WriteLine("hotovo");
                    noChange = true;
                }

                //System.Console.WriteLine("***");
                //return ret;

                count++;
            }

            return ret;
        }

        private int manhatanDist(int x, int y, Point dest)
        {
            return Math.Abs(dest.X-x) + Math.Abs(dest.Y - y);
        }

        
        private int[,] getNeigh(Point point)
        {
            int[,] neigh;
            neigh = new int[4, 3];

            neigh[0, 0] = point.X + this.granularityOfNeigb;
            neigh[0,1] = point.Y;
            neigh[0,2] = -1;

            neigh[1, 0] = point.X - this.granularityOfNeigb;
            neigh[1, 1] = point.Y;
            neigh[1, 2] = -1;

            neigh[2, 0] = point.X;
            neigh[2, 1] = point.Y + this.granularityOfNeigb;
            neigh[2, 2] = -1;

            neigh[3, 0] = point.X;
            neigh[3, 1] = point.Y - this.granularityOfNeigb;
            neigh[3, 2] = -1;

            return neigh;
        }

        


        private void drawMovemenetAreaForPoint(Graphics g, Point p)
        {
            //System.Drawing.SolidBrush myBrush = new System.Drawing.SolidBrush(System.Drawing.Color.LightGray);
            //g.FillRectangle(myBrush, new Rectangle(p.X - movementArea, p.Y - movementArea, movementArea * 2, movementArea * 2));

            g.DrawLine(System.Drawing.Pens.Blue, p.X - movementArea, p.Y - movementArea, p.X + movementArea, p.Y - movementArea);
            g.DrawLine(System.Drawing.Pens.Blue, p.X - movementArea, p.Y - movementArea, p.X - movementArea, p.Y + movementArea);
            g.DrawLine(System.Drawing.Pens.Blue, p.X + movementArea, p.Y + movementArea, p.X + movementArea, p.Y - movementArea);
            g.DrawLine(System.Drawing.Pens.Blue, p.X + movementArea, p.Y + movementArea, p.X - movementArea, p.Y + movementArea);

        }



        private void drawPath(Graphics g, Stack<Point> path)
        {
            Point prevPoint = new Point(0, 0);
            int a = 0;
            foreach (Point p in path)
            {
                if (a == 0 || a - 1 == path.Count)
                {
                    prevPoint = new Point(p.X, p.Y);
                }
                else
                {
                    g.DrawLine(System.Drawing.Pens.Green, prevPoint.X, prevPoint.Y, p.X, p.Y);
                    prevPoint = new Point(p.X, p.Y);
                    g.FillEllipse(new SolidBrush(Color.Green), p.X - 3, p.Y - 3, 6, 6);
                }

                a++;
            }
        }

        // prekoporivanie prvkov z jedneho stacku do druheho stacku 
        private void copyStackToStack(Stack<Point> source, Stack<Point> dest)
        {

            foreach (Point p in source)
            {
                dest.Push(p);
            }

        }

    }
}
