﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Diplo.MentalMap;
using Diplo.Configs;

namespace Diplo.UI.MentalMapModifiers
{
    /// <summary>
    /// Treda, ktorá manipiluje s CONFIG súborom pri nastavovaní parametrov pre KORIDORY / Alfa hrany.
    /// Nastavovanie sa robí z panela nástrojov (časť Visual.UI).
    /// </summary>
    class CoridorMapController
    {
        /// <summary>
        /// <para>zmena pociatovej hodnoty alfa hran</para>
        /// </summary>
        /// <param name="CoridorController"></param>
        /// <param name="mTextBox"></param>
        internal static void alphaChange(CoridorMap CoridorController, TextBox mTextBox)
        {
            double newValue = -150;
            try
            {
                newValue = Convert.ToInt32(mTextBox.Text);
            }
            catch (FormatException exception)
            {
                Debug.MyDebug.print("! POZOR ! Zadalo sa zle číslo" + exception.Source);
            }

            Config.alpha = newValue;
        }

        /// <summary>
        /// <para>zmena movement area</para>
        /// </summary>
        /// <param name="CoridorController"></param>
        /// <param name="mTextBox"></param>
        internal static void movementAreaChange(CoridorMap CoridorController, TextBox mTextBox)
        {
            int newValue = 40;
            try
            {
                newValue = Convert.ToInt32(mTextBox.Text);
            }
            catch (FormatException exception)
            {
                Debug.MyDebug.print("! POZOR ! Zadalo sa zle číslo" + exception.Source);
            }
            Config.movementArea = newValue;
        }

        /// <summary>
        /// <para>zapinanie / vypinanie rozsirovania kodidorov
        /// nastavi sa flag "extendCoridor" v CoridorControllery</para>
        /// </summary>
        /// <param name="CoridorController"></param>
        /// <param name="mButton"></param>
        internal static void extendCoridor(CoridorMap CoridorController, Button mButton)
        {
            if (Config.extendCoridor == true)
            {
                Config.extendCoridor = false;
                mButton.Text = "Zapnut";
            }
            else
            {
                Config.extendCoridor = true;
                mButton.Text = "Vypnut";
            }
        }

        /// <summary>
        /// <para>zapinanie / vypinanie kreslenia alfa kruznic
        /// nastavi sa flag "drawAlphacircles" v CoridorControllery</para>
        /// </summary>
        /// <param name="CoridorController"></param>
        /// <param name="mButton"></param>
        internal static void alphaCircles(CoridorMap CoridorController, Button mButton)
        {
            if (CoridorController.drawAlphacircles == true)
            {
                CoridorController.drawAlphacircles = false;
                mButton.Text = "Zapnut";
            }
            else
            {
                CoridorController.drawAlphacircles = true;
                mButton.Text = "Vypnut";
            }
        }

        /// <summary>
        /// <para>zapinanie / vypinanie kreslenia triangulacnych ciar
        /// nastavi sa flag "drawTriangulation" v CoridorControllery</para>
        /// </summary>
        /// <param name="CoridorController"></param>
        /// <param name="mButton"></param>
        internal static void triangulation(CoridorMap CoridorController, Button mButton)
        {
            if (CoridorController.drawTriangulation == true)
            {
                CoridorController.drawTriangulation = false;
                mButton.Text = "Zapnut";
            }
            else
            {
                CoridorController.drawTriangulation = true;
                mButton.Text = "Vypnut";
            }
        }

    }
}
