﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Diplo.GraphObjects;

namespace Diplo.UI.Visual
{
    /// <summary>
    /// Trieda pre handlovanie Visual.UI - drag and drop mysi.
    /// </summary>
    class HighLighter
    {

        /// <summary>
        /// <para>Vráti zoznam vrcholov, ktoré sa nachádzajú na pozicií myši.</para>
        /// </summary>
        /// <param name="mGraph">Graf.</param>
        /// <param name="mouseX">X súradnica myši.</param>
        /// <param name="mouseY">Y súradnica myši.</param>
        /// <returns></returns>
        public List<Vertex> isInObject(Graph mGraph, int mouseX, int mouseY)
        {
            List<Vertex> underMouse = new List<Vertex>();

            foreach (Vertex mVertex in mGraph.vertices)
            {
                if (
                    (mouseX >= mVertex.pos[0]) &&
                    (mouseX <= (mVertex.pos[0] + mVertex.size)) &&
                    (mouseY >= mVertex.pos[1]) &&
                    (mouseY <= (mVertex.pos[1] + mVertex.size))
                    )
                {
                    mVertex.isUnderMouse = true;
                    underMouse.Add(mVertex);
                }
            }

            return underMouse;
        }

        /// <summary>
        /// <para>Zvýraznenie množiny vrcholov.</para>
        /// </summary>
        /// <param name="g">Grafický komponent.</param>
        /// <param name="mGraph">Graf.</param>
        /// <param name="isUnderMouser">Množina vrcholov z grafu, ktoré budú zvýraznené.</param>
        public void highlight(Graphics g, Graph mGraph, List<Vertex> isUnderMouser)
        {
            foreach (Vertex mVertex in isUnderMouser)
            {
                g.FillEllipse(new SolidBrush(Color.Green), Convert.ToInt32(Math.Round(mVertex.pos[0])),
                    Convert.ToInt32(Math.Round(mVertex.pos[1])), mVertex.size, mVertex.size);
            }

            this.normalise(g, mGraph, isUnderMouser);
        }

        /// <summary>
        /// <para>Nastavenie každému vrcholu, ktorý má byť zvýraznený, parameter IsUnderMouse na TRUE.</para>
        /// </summary>
        /// <param name="g">Grafický komponent.</param>
        /// <param name="mGraph">Graf.</param>
        /// <param name="isUnderMouser">Množina vrcholov z grafu, ktoré budú zvýraznené.</param>
        private void normalise(Graphics g, Graph mGraph, List<Vertex> isUnderMouser)
        {
            foreach (Vertex mVertex in mGraph.vertices)
            {
                Boolean toNormalise = true;

                foreach (Vertex mItemUnderMouse in isUnderMouser)
                {
                    if (
                        mVertex.pos[0] == mItemUnderMouse.pos[0] &&
                        mVertex.pos[1] == mItemUnderMouse.pos[1] &&
                        mVertex.size == mItemUnderMouse.size
                        )
                    {

                        toNormalise = false;
                    }
                }

                if (toNormalise && mVertex.isUnderMouse == true)
                {
                    mVertex.isUnderMouse = false;
                    g.FillEllipse(new SolidBrush(Color.Black), Convert.ToInt32(Math.Round(mVertex.pos[0])),
                    Convert.ToInt32(Math.Round(mVertex.pos[1])), mVertex.size, mVertex.size);
                }

            }
        }

    }
}
