﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.Drawing;

namespace Diplo.UI.Visual
{
    /// <summary>
    /// Vykreslovanie pomocnych textov do pracovnej plochy
    /// </summary>
    class ImageToolTip
    {
        static public String lasHash = "";

        /// <summary>
        /// <para>Vykresli všetky potrebné pomocné texty pri pozicií myši. Vykresľuje len vtedy, ak sa pomocné texty zmenili (ak sú identické, neprekresľujú sa).
        /// V tejto metóde sa rozohoduje, pomocou hashov, či sa má zavolať samotné vykreslenie pomocných textov alebo to nie je potrebné,
        /// </para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="tips">Množina pomocných textov</param>
        /// <param name="mouseX">X súradnica myši</param>
        /// <param name="mouseY">Y súradnica myši</param>
        /// <returns></returns>
        static public Boolean render(Graphics g, List<String> tips, int mouseX, int mouseY)
        {
            if(tips.Count > 0){
                String toHash = "";
                foreach (String s in tips)
                {
                    toHash += s;
                }

                HashAlgorithm algorithm = SHA256.Create();
                byte[] hash = algorithm.ComputeHash(Encoding.UTF8.GetBytes(toHash));

                if (getHashString(hash) == ImageToolTip.lasHash)
                {
                    ImageToolTip.lasHash = getHashString(hash);
                    return false;
                }

                ImageToolTip.lasHash = getHashString(hash);
                draw(g, tips, mouseX, mouseY);
                return true;
            }
            

            return false;
        }

        /// <summary>
        /// <para>Vykreslenie pomocných textov na pozíciu myši.</para>
        /// </summary>
        /// <param name="g">Grafický komponent</param>
        /// <param name="tips">Množina pomocných textov</param>
        /// <param name="mouseX">X súradnica myši</param>
        /// <param name="mouseY">Y súradnica myši</param>
        static private void draw(Graphics g, List<String> tips, int mouseX, int mouseY)
        {

            System.Drawing.Font drawFont = new System.Drawing.Font("Arial", 12);
            System.Drawing.SolidBrush drawBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Black);
            System.Drawing.StringFormat drawFormat = new System.Drawing.StringFormat();

            int toIncrement = 0;
            foreach (String s in tips)
            {
                g.DrawString(s, drawFont, drawBrush, mouseX, mouseY + toIncrement * 15, drawFormat);
                toIncrement++;
            }
        }

        /// <summary>
        /// <para>Z poľa bytov vypočíta HASH.</para>
        /// </summary>
        /// <param name="hash"></param>
        /// <returns></returns>
        private static string getHashString(byte[] hash)
        {
            StringBuilder sb = new StringBuilder();
            foreach (byte b in hash)
                sb.Append(b.ToString("X2"));

            return sb.ToString();
        }

    }
}
